/*
 *  JAJI - Just Another Javatris Implementation.
 *  Copyright (C) 2005  Denis Nelubin aka Gelin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  http://gelin.ru
 *  mailto:java@gelin.ru
 *
 *  2006-03-08  Gelin   Initially created.
 *  2006-04-06  Gelin   Now class it extends AKeyAction
 */

package ru.gelin.games.javatris.actions;

import ru.gelin.games.javatris.game.Javatris;
import ru.gelin.games.javatris.ui.IToolBarModes;
import ru.gelin.games.javatris.util.Messages;
import ru.gelin.games.javatris.util.Images;

import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.KeyStroke;
import java.awt.event.ActionEvent;

/**
 *  This class provides basic action for tool bar buttons.
 *  This action contains information about current show mode of associated button
 *  and it's ability to show text in <code>SHOW_SELECTIVE</code> mode.
 *  @see    IToolBarModes
 */
public abstract class AToolBarAction extends AKeyAction implements IToolBarModes {

    /** The key, used to store current tool bar mode. */
    public static final String TOOL_BAR_MODE = "ToolBarMode";

    /** The key, used to store flag which indicates must button show text in
        <code>SHOW_SELECTIVE</code> mode. */
    public static final String SHOW_TEXT = "ShowText";

    /** messages instance */
    private Messages moMsg = Messages.getInstance();

    /** images instance */
    private Images moImg = Images.getInstance();

    /** key for button text */
    private String msTextKey;

    /** key for button icon */
    private String msIconKey;

    /**
     *  Creates action from all parameters.
     *  @param  aoGame  game reference
     *  @param  asCommand   action command key
     *  @param  aoAccelerators  array of accelerator keys, first element of this array becomes
     *                          default <code>ACCELERATOR_KEY</code>
     *  @param  asText      key of message with button's text
     *  @param  asIcon      key of image
     *  @param  asToolTip   key of message with tooltip text
     *  @param  aiShowMode  current show mode for this button
     *  @param  abShowText  <code>true</code> if button must show text
     *                      in <code>SHOW_SELECTIVE</code> mode
     */
    public AToolBarAction(Javatris aoGame, String asCommand,
            KeyStroke[] aoAccelerators,
            String asText, String asIcon, String asToolTip,
            int aiShowMode, boolean abShowText)
            throws NullPointerException {
        super(asCommand);
        setGame(aoGame);
        setAccelerators(aoAccelerators);
        setText(asText);
        setIcon(asIcon);
        setToolTipText(asToolTip);
        setToolBarMode(aiShowMode);
        setShowText(abShowText);
    }

    /**
     *  Creates action from minimum set of parameters.
     *  Other parameters must be set by appropriate set methods.
     *  @param  asCommand   action command key
     */
    public AToolBarAction(String asCommand) {
        super(asCommand);
    }

    /**
     *  Sets text for button according to provided key.
     *  This method also takes into consideration current show mode and button's
     *  behaviour in <code>SHOW_SELECTIVE</code> mode.
     *  @param  asKey   key in localization bundle
     */
    public void setText(String asKey) {
        msTextKey = asKey;
        String sText = moMsg.get(msTextKey);
        int iShowMode = getToolBarMode();
        if ((iShowMode == SHOW_TEXT_ONLY) ||
                iShowMode == SHOW_ICON_AND_TEXT ||
                ((iShowMode == SHOW_SELECTIVE) && isShowText())) {    //must show in selective mode
            putValue(NAME, sText);
        } else {
            putValue(NAME, null);
        }
    }

    /**
     *  Sets icon for button according to provided key.
     *  This method also takes into consideration current show mode and button's
     *  behaviour in <code>SHOW_SELECTIVE</code> mode.
     *  @param  asKey   key in localization bundle
     */
    public void setIcon(String asKey) {
        msIconKey = asKey;
        ImageIcon oIcon = moImg.get(msIconKey);
        int iShowMode = getToolBarMode();
        if ((iShowMode == SHOW_ICON_ONLY) ||
                (iShowMode == SHOW_ICON_AND_TEXT) ||
                (iShowMode == SHOW_SELECTIVE)) {
            putValue(SMALL_ICON, oIcon);
        } else {
            putValue(SMALL_ICON, null);
        }
    }

    /**
     *  Sets tooltip text for button according to provided key.
     *  @param  asKey   key in localization bundle
     */
    public void setToolTipText(String asKey) {
        String sToolTip = moMsg.get(asKey);
        putValue(SHORT_DESCRIPTION, sToolTip);
    }

    /**
     *  Gets current tool bar mode as integer value. Convenience method.
     *  @return  value from {@link IToolBarModes}
     */
    public int getToolBarMode() {
        Integer oMode = (Integer)(getValue(TOOL_BAR_MODE));
        return oMode != null ? oMode.intValue() : SHOW_ICON_AND_TEXT;
    }

    /**
     *  Sets toolbat mode. Also changes visibility of button icon and text.
     *  @param  aiMode  value from {@link IToolBarModes}
     */
    public void setToolBarMode(int aiMode) {
        putValue(TOOL_BAR_MODE, new Integer(aiMode));
        setText(msTextKey);
        setIcon(msIconKey);
    }

    /**
     *  Is button must show text in <code>SHOW_SELECTIVE</code> mode?
     *  Convenience method.
     *  @return  <code>true</code> if button must show text
     *              in <code>SHOW_SELECTIVE</code> mode
     */
    public boolean isShowText() {
        Boolean oShowText = (Boolean)(getValue(SHOW_TEXT));
        return oShowText != null ? oShowText.booleanValue() : true;
    }

    /**
     *  Sets button's behaviour in <code>SHOW_SELECTIVE</code> mode.
     *  @param  abShowText  <code>true</code> if button must show text
     *                      in <code>SHOW_SELECTIVE</code> mode
     */
    public void setShowText(boolean abShowText) {
        putValue(SHOW_TEXT, new Boolean(abShowText));
        setText(msTextKey);
        setIcon(msIconKey);
    }

    /**
     *  This method is called then action is performed.
     */
    public abstract void actionPerformed(ActionEvent aoEvent);

}