/*
 *  JAJI - Just Another Javatris Implementation.
 *  Copyright (C) 2005  Denis Nelubin aka Gelin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  http://gelin.ru
 *  mailto:java@gelin.ru
 *
 *  2005-10-29  Gelin   Initially created
 */

package ru.gelin.games.javatris.game;

import ru.gelin.games.javatris.ui.ColorTableEvent;

import java.awt.Color;
import java.util.Collections;

/**
 *  Block.
 */
public class Block extends AColorTableModel implements IBlockColors, Cloneable {

    /** size of internal block representation */
    static final int SIZE = 4;

    /** border around block representation, in cells */
    static private final int BORDER = 1;

    /** Width of block representation */
    private int miWidth = SIZE + BORDER * 2;

    /** Height of block representation */
    private int miHeight = SIZE + BORDER * 2;

    /** type of this block */
    private int miType = MIN_BLOCK_TYPE;

    /** game which this block belongs to */
    private Javatris moGame;

    /** default cell size */
    static private final int PREFERRED_CELL_SIZE = 20;

    /** minimum cell size */
    static private final int MINIMUM_CELL_SIZE = 5;

    /** internal block representation */
    private int[][] miaBlock = new int[SIZE][SIZE];

    /** position of block center */
    private int miCenterX = 1;

    /**
     *  Constructor.
     *  @param  aoGame  game reference, block is created for real game
     */
    public Block(Javatris aoGame) {
        super();
        moGame = aoGame;
        // created random block
        miType = moGame.getRandom().nextInt(MAX_BLOCK_TYPE - MIN_BLOCK_TYPE + 1) + MIN_BLOCK_TYPE;
        init();
        //rotate();
    }

    /**
     *  Returns color contained in specified cell.
     */
    public Color getColor(int aiX, int aiY) {
        Color oResult = null;
        // background
        if (((aiX + aiY) % 2) == 0) {
            oResult = moGame.getSettings().getColor(BG_COLOR_1);
        } else {
            oResult = moGame.getSettings().getColor(BG_COLOR_2);
        }
        aiX -= BORDER;
        aiY -= BORDER;
        if (aiX >= 0 && aiX < SIZE && aiY >= 0 && aiY < SIZE &&
                miaBlock[aiX][aiY] != BLOCK_EMPTY) {
            oResult = moGame.getSettings().getColor(miaBlock[aiX][aiY]);
        }
        return oResult;
    }

    /**
     *  Returns width of block representation.
     */
    public int getWidth() {
        return miWidth;
    }

    /**
     *  Returns height of block representation.
     */
    public int getHeight() {
        return miHeight;
    }

    public int getPreferredCellSize() {
        return PREFERRED_CELL_SIZE;
    }

    public int getMinimumCellSize() {
        return MINIMUM_CELL_SIZE;
    }

    public int getMaximumCellSize() {
        return PREFERRED_CELL_SIZE;
    }

    /**
     *  Returns X coordinate of block center;
     */
    public int getCenterX() {
        return miCenterX;
    }

    /**
     *  Returns internal array representation.
     *  Package access, used in {@link Glass}.
     */
    int[][] getArray() {
        return miaBlock;
    }

    /**
     *  Prepare block, fill internal representation.
     */
    private void init() {
        for (int x = 0; x < SIZE; x++) {
            for (int y = 0; y < SIZE; y++) {
                miaBlock[x][y] = BLOCK_EMPTY;
            }
        }
        switch (miType) {
            case BLOCK_I:
                miaBlock[1][0]=miaBlock[1][1]=miaBlock[1][2]=miaBlock[1][3]=BLOCK_I;
                miCenterX = 1;
            break;
            case BLOCK_L:
                miaBlock[1][0]=miaBlock[1][1]=miaBlock[1][2]=miaBlock[2][2]=BLOCK_L;
                miCenterX = 1;
            break;
            case BLOCK_IL:
                miaBlock[2][0]=miaBlock[2][1]=miaBlock[2][2]=miaBlock[1][2]=BLOCK_IL;
                miCenterX = 2;
            break;
            case BLOCK_T:
                miaBlock[1][0]=miaBlock[1][1]=miaBlock[1][2]=miaBlock[2][1]=BLOCK_T;
                miCenterX = 1;
            break;
            case BLOCK_Z:
                miaBlock[2][0]=miaBlock[1][1]=miaBlock[2][1]=miaBlock[1][2]=BLOCK_Z;
                miCenterX = 1;
            break;
            case BLOCK_S:
                miaBlock[1][0]=miaBlock[1][1]=miaBlock[2][1]=miaBlock[2][2]=BLOCK_S;
                miCenterX = 2;
            break;
            case BLOCK_SQ:
                miaBlock[1][1]=miaBlock[2][1]=miaBlock[1][2]=miaBlock[2][2]=BLOCK_SQ;
                miCenterX = 1;
            break;
        }
    }

    /**
     *  Rotate block.
     */
    public void rotate() {
        int[][] iaBlock = new int[SIZE][SIZE];    //temporary array
        int i, j;
        int max = SIZE;  //trans matrix size
        if(miType == BLOCK_T) max = SIZE - 1;
        if(!moGame.getSettings().isRotationClockwise()) {
            for (i = 0; i < max; i++) {
                for(j = 0; j < max; j++) {
                    iaBlock[j][max - 1 - i] = miaBlock[i][j];
                }
            }
        } else {
            for (i = 0; i < max; i++) {
                for (j = 0; j < max; j++) {
                    iaBlock[max - 1 - j][i] = miaBlock[i][j];
                }
            }
        }
        for (i = 0; i < max; i++) {
            for (j =0; j < max; j++) {
                miaBlock[i][j] = iaBlock[i][j];
            }
        }
        // fire events
        fireColorTableEvent(new ColorTableEvent(this));
    }

    /**
     *  Clones block.
     *  @return new <code>Block</code> with the same content as this block
     */
    public Object clone() {
        Block oBlock = new Block(moGame);
        oBlock.miWidth = miWidth;
        oBlock.miHeight = miHeight;
        oBlock.miType = miType;
        for (int i = 0; i < SIZE; i++) {
            for (int j = 0; j < SIZE; j++) {
                oBlock.miaBlock[i][j] = miaBlock[i][j];
            }
        }
        //Collections.copy(oBlock.moListeners, moListeners);
        oBlock.moListeners = moListeners;
        return oBlock;
    }

}
