/*
 *  JAJI - Just Another Javatris Implementation.
 *  Copyright (C) 2005  Denis Nelubin aka Gelin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  http://gelin.ru
 *  mailto:java@gelin.ru
 *
 *  2005-10-24  Gelin   Initially created
 *  2006-08-26  Gelin   Added moving block by mouse events
 */

package ru.gelin.games.javatris.game;

import ru.gelin.games.javatris.ui.ColorTableEvent;

import java.awt.Color;
import java.awt.Rectangle;

/**
 *  Glass for blocks.
 */
public class Glass extends AColorTableModel implements IBlockColors {

    /** Width of glass */
    private int miWidth = 12;

    /** Height of glass */
    private int miHeight = 19;

    /** game which this glass belongs to */
    private Javatris moGame;

    /** internal glass representation */
    private int[][] miaGlass = new int[miWidth][miHeight];

    /** block which is now in glass */
    private Block moBlock = null;

    /** block horizontal position */
    private int miBlockX;

    /** block vertical position */
    private int miBlockY;

    /** default cell size */
    static private final int PREFERRED_CELL_SIZE = 20;

    /** minimum cell size */
    static private final int MINIMUM_CELL_SIZE = 5;

    /**
     *  Constructor.
     *  @param  aoGame  game reference, glass is created for real game
     */
    public Glass(Javatris aoGame) {
        super();
        moGame = aoGame;
    }

    /**
     *  Returns color contained in specified cell.
     */
    public Color getColor(int aiX, int aiY) {
        Color oResult = null;
        // background
        if (((aiX + aiY) % 2) == 0) {
            oResult = moGame.getSettings().getColor(IBlockColors.BG_COLOR_1);
        } else {
            oResult = moGame.getSettings().getColor(IBlockColors.BG_COLOR_2);
        }
        if (isInGlass(aiX, aiY) && miaGlass[aiX][aiY] != BLOCK_EMPTY) {
            oResult = moGame.getSettings().getColor(miaGlass[aiX][aiY]);
        }
        return oResult;
    }

    /**
     *  Returns glass width.
     */
    public int getWidth() {
        return miWidth;
    }

    /**
     *  Returns glass height.
     */
    public int getHeight() {
        return miHeight;
    }

    public int getPreferredCellSize() {
        return PREFERRED_CELL_SIZE;
    }

    public int getMinimumCellSize() {
        return MINIMUM_CELL_SIZE;
    }

    public int getMaximumCellSize() {
        return PREFERRED_CELL_SIZE;
    }

    /**
     *  Clears glass, prepare for new game
     */
    public void clear() {
        for (int i = 0; i < miaGlass.length; i++) {
            for (int j = 0; j < miaGlass[i].length; j++) {
                miaGlass[i][j] = BLOCK_EMPTY;
            }
        }
        moBlock = null;
        fireColorTableEvent(new ColorTableEvent(this));
    }

    /**
     *  Puts new block to glass.
     *  @param  aoBlock block to be inserted into glass
     *  @return <code>true</code> if block successfully inserted,
     *          <code>false</code> if glass is full and block can't be added
     *  @throws IllegalStateException   if another block is already in glass.
     *                  You must {@link #plant} block before insert new one.
     */
    public boolean put(Block aoBlock) throws IllegalStateException {
        if (moBlock != null) {
            throw new IllegalStateException("plant() block before put() new one");
        }
        // check if block can be inserted
        int iX = miWidth / 2 - Block.SIZE / 2;  // block may be inserted here
        for (int i = 0; i < Block.SIZE; i++) {
            for (int j = 0; j < Block.SIZE; j++) {
                if (aoBlock.getArray()[i][j] != BLOCK_EMPTY &&
                        miaGlass[i + iX][j] != BLOCK_EMPTY) {
                    return false;   // new block overwrites non empty cells, can't be inserted
                }
            }
        }
        // insert new block
        moBlock = aoBlock;
        miBlockX = iX;
        miBlockY = 0;
        for (int i = 0; i < Block.SIZE; i++) {
            for (int j = 0; j < Block.SIZE; j++) {
                if (moBlock.getArray()[i][j] != BLOCK_EMPTY) {
                    miaGlass[i + miBlockX][j + miBlockY] = moBlock.getArray()[i][j];
                }
            }
        }
        // announce listeners
        fireColorTableEvent(new ColorTableEvent(this, new Rectangle(miBlockX, miBlockY, Block.SIZE, Block.SIZE)));
        return true;
    }

    /**
     *  Moves inserted block one line down.
     *  @return <code>true</code> if block successfully moved
     *          <code>false</code> if block can't be moved down
     *  @throws IllegalStateException   if there is no block in glass, call {@link #put}
     */
    public boolean moveDown() throws IllegalStateException {
        if (moBlock == null) {
            throw new IllegalStateException("glass is empty, put() block to it");
        }
        return move(0, 1);
    }

    /**
     *  Moves inserted block one cell left.
     *  @return <code>true</code> if block successfully moved
     *          <code>false</code> if block can't be moved left
     *  @throws IllegalStateException   if there is no block in glass, call {@link #put}
     */
    public boolean moveLeft() throws IllegalStateException {
        if (moBlock == null) {
            throw new IllegalStateException("glass is empty, put() block to it");
        }
        return move(-1, 0);
    }

    /**
     *  Moves inserted block one cell right.
     *  @return <code>true</code> if block successfully moved
     *          <code>false</code> if block can't be moved right
     *  @throws IllegalStateException   if there is no block in glass, call {@link #put}
     */
    public boolean moveRight() throws IllegalStateException {
        if (moBlock == null) {
            throw new IllegalStateException("glass is empty, put() block to it");
        }
        return move(1, 0);
    }

    /**
     *  Rotates block.
     *  @return <code>true</code> if block successfully rotated
     *          <code>false</code> if block can't be rotated
     *  @throws IllegalStateException   if there is no block in glass, call {@link #put}
     */
    public boolean rotate() throws IllegalStateException {
        if (moBlock == null) {
            throw new IllegalStateException("glass is empty, put() block to it");
        }
        boolean bResult = true; // block can be rotated by default
        // create new rotated block
        Block oRotated = (Block)(moBlock.clone());
        oRotated.rotate();
        // clear old block location
        for (int i = 0; i < Block.SIZE; i++) {
            for (int j = 0; j < Block.SIZE; j++) {
                if (moBlock.getArray()[i][j] != BLOCK_EMPTY) {
                    miaGlass[i + miBlockX][j + miBlockY] = BLOCK_EMPTY;
                }
            }
        }
        // find in which column rotation is stopped by previous blocks
        boolean bMovingRight = false;
        boolean bMovingLeft = false;
        for (int i = 0; i < Block.SIZE; i++) {
            for (int j = 0; j < Block.SIZE; j++) {
                if (oRotated.getArray()[i][j] != BLOCK_EMPTY &&
                        (!isInGlass(i + miBlockX, j + miBlockY) ||
                        miaGlass[i + miBlockX][j + miBlockY] != BLOCK_EMPTY)) {
                    if (i < Block.SIZE / 2) {
                        bMovingRight = true;    // stopped by left cells, try to move right
                    } else {
                        bMovingLeft = true; // stopped by right cells, try to move left
                    }
                    bResult = false;    // rotation is blocked
                }
            }
        }
        // try to adjust block position after rotation
        int iDX = 0;    // horizontal adjustment
        if(bMovingRight && !bMovingLeft) {
            // adjust for one cell right
            bResult = true; // suppose adjustment will be successfull
            for (int i = 0; i < Block.SIZE; i++) {
                for (int j = 0; j < Block.SIZE; j++) {
                    if (oRotated.getArray()[i][j] != BLOCK_EMPTY &&
                            (!isInGlass(i + miBlockX + 1, j + miBlockY) ||
                            miaGlass[i + miBlockX + 1][j + miBlockY] != BLOCK_EMPTY)) {
                        bResult = false;    // adjustment unsuccessfull
                    }
                }
            }
            if (bResult) {
                iDX = 1;
            } else {
                // adjust for two cells right
                bResult = true; // suppose adjustment will be successfull
                for (int i = 0; i < Block.SIZE; i++) {
                    for (int j = 0; j < Block.SIZE; j++) {
                        if (oRotated.getArray()[i][j] != BLOCK_EMPTY &&
                                (!isInGlass(i + miBlockX + 2, j + miBlockY) ||
                                miaGlass[i + miBlockX + 2][j + miBlockY] != BLOCK_EMPTY)) {
                            bResult = false;    // adjustment unsuccessfull
                        }
                    }
                }
                if (bResult) {
                    iDX = 2;
                }
            }
        } else if (bMovingLeft && !bMovingRight) {
            // adjust for one cell left
            bResult = true; // suppose adjustment will be successfull
            for (int i = 0; i < Block.SIZE; i++) {
                for (int j = 0; j < Block.SIZE; j++) {
                    if (oRotated.getArray()[i][j] != BLOCK_EMPTY &&
                            (!isInGlass(i + miBlockX - 1, j + miBlockY) ||
                            miaGlass[i + miBlockX - 1][j + miBlockY] != BLOCK_EMPTY)) {
                        bResult = false;    // adjustment unsuccessfull
                    }
                }
            }
            if (bResult) {
                iDX = -1;
            } else {
                // adjust for two cells left
                bResult = true; // suppose adjustment will be successfull
                for (int i = 0; i < Block.SIZE; i++) {
                    for (int j = 0; j < Block.SIZE; j++) {
                        if (oRotated.getArray()[i][j] != BLOCK_EMPTY &&
                                (!isInGlass(i + miBlockX - 2, j + miBlockY) ||
                                miaGlass[i + miBlockX - 2][j + miBlockY] != BLOCK_EMPTY)) {
                            bResult = false;    // adjustment unsuccessfull
                        }
                    }
                }
                if (bResult) {
                    iDX = -2;
                }
            }
        }
        // update block form and position
        if (bResult) {
            miBlockX += iDX;
            moBlock = oRotated;
        }
        // draw block on new or old position
        for (int i = 0; i < Block.SIZE; i++) {
            for (int j = 0; j < Block.SIZE; j++) {
                if (isInGlass(i + miBlockX, j + miBlockY) &&
                        moBlock.getArray()[i][j] != BLOCK_EMPTY) {
                    miaGlass[i + miBlockX][j + miBlockY] = moBlock.getArray()[i][j];
                }
            }
        }
        // announce listeners
        if (bResult) {
            fireColorTableEvent(new ColorTableEvent(this,
                    new Rectangle(miBlockX - 2, miBlockY, Block.SIZE + 4, Block.SIZE)));
        }
        return bResult;
    }

    /**
     *  Fixes block position ("plant" block) and removes filled lines.
     *  After call to this method glass is ready to {@link #put} next block.
     *  @return number of filled rows which were removed
     *  @throws IllegalStateException   if there is no block in glass, call {@link #put}
     */
    public int plant() throws IllegalStateException {
        if (moBlock == null) {
            throw new IllegalStateException("glass is empty, put() block to it");
        }
        int iLines = 0; // filled lines counter
        // count and delete filled lines
        for (int j = 0; j < miHeight; j++) {    // for each line
            boolean bFilled = true;  // line is filled
            for (int i = 0; i < miWidth; i++) {
                if (miaGlass[i][j] == BLOCK_EMPTY) {
                    bFilled = false;    //line is not filled
                    break;
                }
            }
            if (bFilled) {
                deleteLine(j);
                iLines++;
            }
        }
        // announce listeners
        //if (iLines != 0) {
        //    fireColorTableEvent(new ColorTableEvent(this));
        //}
        // current block is "planted"
        moBlock = null;
        return iLines;
    }

    /**
     *  Checks if block was {@link #put}'ted into glass or not.
     *  @return <code>true</code> if new block can be putted, glass is empty
     *          <code>false</code> if block is not {@link #plant}'ed, new block can't be putted
     */
    public boolean isEmpty() {
        return moBlock == null;
    }

    /**
     *  Moves block to specified X coordinate. Used for moving block by mouse events.
     */
    public void moveTo(int aiX) {
        aiX -= moBlock.getCenterX();
        while (aiX != miBlockX &&
            move(aiX < miBlockX ? -1 : 1, 0));
    }

    /**
     *  Moves inserted block to specified direction.
     *  @param  aiDX    horizontal offset to move block
     *  @param  aiDY    vertical offset to move block
     *  @return <code>true</code> if block successfully moved
     *          <code>false</code> if block can't be moved
     */
    private boolean move(int aiDX, int aiDY) {
        boolean bResult = true; // by default block can be moved down
        // clear previous block location
        for (int i = 0; i < Block.SIZE; i++) {
            for (int j = 0; j < Block.SIZE; j++) {
                if (moBlock.getArray()[i][j] != BLOCK_EMPTY) {
                    miaGlass[i + miBlockX][j + miBlockY] = BLOCK_EMPTY;
                }
            }
        }
        // check if block can be moved
        for (int i = 0; i < Block.SIZE; i++) {
            for(int j = 0; j < Block.SIZE; j++) {
                if (moBlock.getArray()[i][j] != BLOCK_EMPTY &&  // block cell is not empty and...
                        (!isInGlass(i + miBlockX + aiDX, j + miBlockY + aiDY) ||    // block tryes to go away from glass or...
                        miaGlass[i + miBlockX + aiDX][j + miBlockY + aiDY] != BLOCK_EMPTY))   // previous blocks stop this one
                {
                    bResult = false;    // can't move
                }
            }
        }
        // update block location
        if (bResult) {
            miBlockX += aiDX;
            miBlockY += aiDY;
        }
        // draw block on new or old location
        for (int i = 0; i < Block.SIZE; i++) {
            for(int j = 0; j < Block.SIZE; j++) {
                if (isInGlass(i + miBlockX, j + miBlockY) &&
                        moBlock.getArray()[i][j] != BLOCK_EMPTY) {
                    miaGlass[i + miBlockX][j + miBlockY] = moBlock.getArray()[i][j];
                }
            }
        }
        // announce listeners
        if (bResult) {
            fireColorTableEvent(new ColorTableEvent(this,
                    new Rectangle(miBlockX - 1, miBlockY - 1, Block.SIZE + 2, Block.SIZE + 1)));
        }
        return bResult;
    }

    /**
     *  Checks if this coordinates are into glass.
     *  @param  aiX X coordinate for checking
     *  @param  aiY Y coordinate for checking
     *  @return <code>true</code> if provided coordinates are in glass ones
     */
    private boolean isInGlass(int aiX, int aiY) {
        return (aiX >=0 && aiX < miWidth && aiY >=0 && aiY < miHeight);
    }

    /**
     *  Deletes one filled line from glass.
     *  @param  aiLine  Y coordinate of deleting line
     */
    private void deleteLine(int aiLine) {
        // shift top lines down
        for(int j = aiLine - 1; j >= 0; j--) {
            for(int i = 0; i < miWidth; i++) {
                miaGlass[i][j + 1] = miaGlass[i][j];
            }
        }
        // clear first line
        for(int i = 0; i < miWidth; i++) {
            miaGlass[i][0] = BLOCK_EMPTY;
        }
        // announce listeners
        fireColorTableEvent(new ColorTableEvent(this));
    }

}
