/*
 *  JAJI - Just Another Javatris Implementation.
 *  Copyright (C) 2005  Denis Nelubin aka Gelin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  http://gelin.ru
 *  mailto:java@gelin.ru
 *
 *  2005-10-06  Gelin   Initially created
 *  ...
 *  2006-04-06  Gelin   Added pause
 *  2006-05-26  Gelin   Added saving/loading
 *  2006-08-26  Gelin   Bugfix #3. Added method isBlockStopped()
 */

package ru.gelin.games.javatris.game;

import ru.gelin.games.javatris.ui.IColorTableModel;
import ru.gelin.games.javatris.ui.IColorTableListener;
import ru.gelin.games.javatris.ui.ColorTableEvent;
import ru.gelin.games.javatris.ui.ColorTable;
import ru.gelin.games.javatris.ui.NewRecordDialog;
import ru.gelin.games.javatris.actions.IActionNames;
import ru.gelin.games.javatris.actions.TimerAction;
import ru.gelin.games.javatris.actions.MoveLeftAction;
import ru.gelin.games.javatris.actions.MoveRightAction;
import ru.gelin.games.javatris.actions.RotateAction;
import ru.gelin.games.javatris.actions.DropAction;
import ru.gelin.games.javatris.actions.NewGameAction;
import ru.gelin.games.javatris.actions.AToggleToolBarAction;
import ru.gelin.games.javatris.actions.PauseGameAction;
import ru.gelin.games.javatris.actions.SettingsAction;
import ru.gelin.games.javatris.actions.Top10Action;
import ru.gelin.games.javatris.actions.AboutAction;
import ru.gelin.games.javatris.util.Messages;
import ru.gelin.games.javatris.util.Images;

import java.util.Random;
import java.util.Map;
import java.util.HashMap;
import java.util.Collection;
import java.util.Collections;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.awt.Color;
import javax.swing.Timer;
import javax.swing.Action;

/**
 *  Base class implementing game logic.
 */
public class Javatris implements IActionNames {

    /** game version */
    public static final String VERSION;

    /** game build */
    public static final String BUILD;

    /** glass */
    private Glass moGlass = new Glass(this);

    /** current game score */
    private Counter moScore = new Counter();

    /** current game lines */
    private Counter moLines = new Counter();

    /** current game level */
    private Counter moLevel = new LevelCounter();

    /** next block */
    private Block moNextBlock = null;
    /** next block wrapper */
    private NextBlockWrapper moNextBlockWrapper = new NextBlockWrapper();

    /** random source of game */
    private Random moRandom = null;

    /** settings of this game */
    private Settings moSettings = new Settings();

    /** actions map: string action name to action object */
    private Map moActions = new HashMap();

    /** main game timer */
    private Timer moTimer = null;

    /** indicates that block can't be moved down more */
    private boolean mbBlockStopped = false;

    /** indicates that block is dropping down */
    private boolean mbDropping = false;

    /** initial game level */
    private int miInitLevel;
    // this level must be saved because settings can be changed during a game

    /** paused by direct manipulation flag */
    private boolean mbPausedManual = false;

    /** paused by indirect manipulation flag */
    private boolean mbPausedAuto = false;

    /** default player name */
    private String msDefPlayer = "";

    /** game saver */
    private ISaver moSaver;

    /** logger name */
    static public final String LOGGER_NAME = "ru.gelin.games.javatris.game.Javatris";
    /** default log level */
    static public final Level DEF_LOG_LEVEL = Level.WARNING;
    /** logger instance */
    static private Logger moLog = Logger.getLogger(LOGGER_NAME);

    /** messages instance */
    static private Messages moMsg = Messages.getInstance();
    /** images instance */
    static private Images moImg = Images.getInstance();

    /**
     *  Creates game, random source is init internally, no saver.
     */
    public Javatris() {
        this(new Random(), null);
    }

    /**
     *  Creates game with specified saver, random source is init internally.
     *  @param  aoSaver game saver
     */
    public Javatris(ISaver aoSaver) {
        this(new Random(), aoSaver);
    }

    /**
     *  Creates game with specified random source, no saver.
     *  @param  aoRandom    external random source
     */
    public Javatris(Random aoRandom) {
        this(aoRandom, null);
    }

    /**
     *  Creates game, requests external random source and game saver.
     *  @param  aoRandom    external provided random source.
     *  @param  aoSaver     game saver
     */
    public Javatris(Random aoRandom, ISaver aoSaver) {
        moRandom = aoRandom;
        moSaver = aoSaver;
        setNextBlock(new Block(this));
        initActions();
        load(); //load game settings
        moTimer = new Timer(moSettings.getTimerDelay(moLevel.get()), getAction(TIMER_ACTION));
        try {
            msDefPlayer = System.getProperty("user.name");
        } catch(SecurityException se) {
            moLog.logp(Level.INFO, "Javatris", "<init>", "Can't get user name.", se);
        }
        startNewGame();
    }

    /**
     *  Returns game glass.
     */
    public Glass getGlass() {
        return moGlass;
    }

    /**
     *  Returns current game score.
     */
    public Counter getScore() {
        return moScore;
    }

    /**
     *  Returns current game lines.
     */
    public Counter getLines() {
        return moLines;
    }

    /**
     *  Returns current game level.
     *  @see    IGameLevels
     */
    public Counter getLevel() {
        return moLevel;
    }

    /**
     *  Returns next block.
     *  Don't use this method to get next block as {@link IColorTableModel} for UI. Use {@link #getNextBlockModel}
     *  instead.
     */
    Block getNextBlock() {
        return moNextBlock;
    }

    /**
     *  Creates and returns wrapper for next block. Next {@link Block} directly implements {@link IColorTableModel}, but
     *  reference to next block can't be used as model for {@link ColorTable} because it can be modified during the game.
     *  Object returned by this method is constant reference implemented <code>IColorTableModel</code>, but it is constant.
     *  This object relies to <code>IColorTableModel</code> methods of current next block.
     */
    public IColorTableModel getNextBlockModel() {
        return moNextBlockWrapper;
    }

    /**
     *  Sets new next block. Should use this method instead direct assigning for wrapper correctly working.
     */
    private void setNextBlock(Block aoBlock) {
        moNextBlock = aoBlock;
        moNextBlock.addColorTableListener(moNextBlockWrapper);
        moNextBlockWrapper.colorTableChanged(new ColorTableEvent(moNextBlockWrapper));
    }

    /**
     *  Wrapper for new block model. It's need because next block reference can be changed, but reference to model shouldn't
     *  be change.
     */
    private class NextBlockWrapper extends AColorTableModel implements IColorTableListener {
        public Color getColor(int aiX, int aiY) {
            if (moSettings.isBlockPreview()) {
                return moNextBlock.getColor(aiX, aiY);
            } else {
                return moSettings.getColor(IBlockColors.BG_COLOR_1);
            }
        }
        public int getWidth() {
            return moNextBlock.getWidth();
        }
        public int getHeight() {
            return moNextBlock.getHeight();
        }
        public int getPreferredCellSize() {
            return moNextBlock.getPreferredCellSize();
        }
        public int getMinimumCellSize() {
            return moNextBlock.getMinimumCellSize();
        }
        public int getMaximumCellSize() {
            return moNextBlock.getMaximumCellSize();
        }
        public void colorTableChanged(ColorTableEvent aoEvent) {
            fireColorTableEvent(aoEvent);
        }
    }

    /**
     *  Returns random generator of this game.
     */
    Random getRandom() {
        return moRandom;
    }

    /**
     *  Returns settings of this game.
     */
    public Settings getSettings() {
        return moSettings;
    }

    /**
     *  Returns game action by name.
     *  @param  asName  name of the action for lookup
     *  @return found action or <code>null</code> if action is not found
     */
    public Action getAction(String asName) {
        return (Action)(moActions.get(asName));
    }

    /**
     *  Returns collection of all possible game actions.
     */
    public Collection getActions() {
        return Collections.unmodifiableCollection(moActions.values());
    }

    /**
     *  This method is called every timer tick.
     */
    public void timerTick() {
        if (mbBlockStopped) {
            // fix last block
            int iLines = moGlass.plant();
            // increment score and lines
            moLines.add(iLines);
            moScore.add(moSettings.getScoreForLines(iLines));
            setNextLevel();
            // try to add next block
            if (!putNextBlock()) {
                // add new hero...
                startNewGame();
            }
        } else {
            // block ordinary goes down
            if (moGlass.isEmpty()) {
                putNextBlock();
            } else {
                // block goes down
                mbBlockStopped = !moGlass.moveDown();
                if (mbDropping) {
                    moScore.add(moSettings.getScoreForDrop());  // increment score while dropping
                    if (mbBlockStopped) {
                        stopDrop(); // block dropped, stop dropping
                    }
                }
            }
        }
    }

    /**
     *  Starts new game.
     */
    public void startNewGame() {
        moTimer.stop();
        if (moSettings.getRecords().isNewRecord(moScore.get())) {
            insertNewRecord();
        }
        setNextBlock(new Block(this));
        moGlass.clear();
        putNextBlock();
        moScore.set(0);
        moLines.set(0);
        miInitLevel = moSettings.getNewLevel();
        moLevel.set(miInitLevel);
        moTimer.setDelay(moSettings.getTimerDelay(moLevel.get()));
        resume();
    }

    /**
     *  Shows dialog asking player's name and inserts new record into records list.
     */
    private void insertNewRecord() {
        NewRecordDialog oDialog = new NewRecordDialog(msDefPlayer);
        String sPlayer = oDialog.show();
        if (sPlayer != null) {
            msDefPlayer = sPlayer;
            RecordItem oRecord = new RecordItem(sPlayer, moScore.get());
            RecordsList oRecords = moSettings.getRecords();
            oRecords.put(oRecord);
            oRecords.commit();
            save();
        }
    }

    /**
     *  Drops block to the glass' bottom.
     *  'Dropping' is just moving down with minimal timer delay.
     */
    public void dropBlock() {
        mbDropping = true;
        moTimer.setDelay(moSettings.getDropDelay());
        // otherwise delay don't changes immediately
        moTimer.setInitialDelay(moSettings.getDropDelay());
        moTimer.restart();
    }

    /**
     *  Pauses game by direct user manipulation.
     */
    public void pauseManual() {
        mbPausedManual = true;
        pause();
    }

    /**
     *  Pauses game by indirect manipulation, for example, by minimizing window.
     */
    public void pauseAuto() {
        mbPausedAuto = true;
        pause();
    }

    /**
     *  Resumes game by direct user manipulation.
     */
    public void resumeManual() {
        resume();
    }

    /**
     *  Resumes game by indirect manipulation, for example, by restoring window.
     */
    public void resumeAuto() {
        if (!mbPausedManual) {
            resume();
        }
    }

    /**
     *  Checks pause state of the game.
     */
    public boolean isPaused() {
        return mbPausedManual || mbPausedAuto;
    }

    /**
     *  Checks if block is stopped in dropping down.
     */
    public boolean isBlockStopped() {
        return mbBlockStopped;
    }

    /**
     *  Saves game state.
     */
    public void save() {
        if (moSaver != null) {
            moSaver.save(moSettings);
            moSaver.save(moSettings.getRecords());
        }
    }

    /**
     *  Loads game state.
     */
    public void load() {
        if (moSaver != null) {
            moSaver.load(moSettings);
            moSaver.load(moSettings.getRecords());
        }
    }

    /**
     *  Sets game saver.
     *  @param  aoSaver game saver
     */
    public void setSaver(ISaver aoSaver) {
        moSaver = aoSaver;
    }

    /**
     *  Stops droppping process, when block reaches glass bottom.
     */
    private void stopDrop() {
        mbDropping = false;
        moTimer.setDelay(moSettings.getTimerDelay(moLevel.get()));
        //moTimer.setInitialDelay(moSettings.getDropDelay());
        //moTimer.restart();
    }

    /**
     *  Creates all game actions and fills action map.
     */
    private void initActions() {
        moActions.put(TIMER_ACTION, new TimerAction(this));
        moActions.put(MOVE_LEFT_ACTION, new MoveLeftAction(this));
        moActions.put(MOVE_RIGHT_ACTION, new MoveRightAction(this));
        moActions.put(ROTATE_ACTION, new RotateAction(this));
        moActions.put(DROP_ACTION, new DropAction(this));
        moActions.put(NEW_GAME_ACTION, new NewGameAction(this));
        moActions.put(PAUSE_GAME_ACTION, new PauseGameAction(this));
        moActions.put(SETTINGS_ACTION, new SettingsAction(this));
        moActions.put(TOP10_ACTION, new Top10Action(this));
        moActions.put(ABOUT_ACTION, new AboutAction(this));
    }

    /**
     *  Puts next block into glass, generated new next block.
     *  @return <code>true</code> if block successfully added
     *          <code>false</code> if block can't be added
     */
    private boolean putNextBlock() {
        boolean bResult = moGlass.put(moNextBlock);
        if (bResult) {
            setNextBlock(new Block(this));
            mbBlockStopped = false;
            mbDropping = false;
            moScore.add(moSettings.getScoreForBlock());
        }
        return bResult;
    }

    /**
     *  Sets next game level according to current filled lines number.
     */
    private void setNextLevel() {
        int iNextLevel = moLines.get() / moSettings.getLinesForLevel() +
                miInitLevel;
        moLevel.set(iNextLevel);
        moTimer.setDelay(moSettings.getTimerDelay(moLevel.get()));
    }

    /**
     *  Pauses game.
     */
    private void pause() {
        moTimer.stop();
        ((AToggleToolBarAction)getAction(PAUSE_GAME_ACTION)).setSelected(true);
    }

    /**
     *  Resumes game after pause.
     */
    private void resume() {
        mbPausedManual = false;
        mbPausedAuto = false;
        moTimer.start();
        ((AToggleToolBarAction)getAction(PAUSE_GAME_ACTION)).setSelected(false);
    }

    /**
     *  Loads current version and build from properties file
     */
    static {
        String sVersion = "unknown";
        String sBuild = "unknown";
        try {
            ResourceBundle oBundle = ResourceBundle.getBundle("ru.gelin.games.javatris.Version");
            sVersion = oBundle.getString("version");
            sBuild = oBundle.getString("build");
        } catch(MissingResourceException e) {
            System.err.println("Javatris: Can't load file version properties");
            System.err.println(e);
        } finally {
            VERSION = sVersion;
            BUILD = sBuild;
        }
    }

}
