/*
 *  JAJI - Just Another Javatris Implementation.
 *  Copyright (C) 2005  Denis Nelubin aka Gelin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  http://gelin.ru
 *  mailto:java@gelin.ru
 *
 *  2006-01-14  Gelin   Initially created.
 */

package ru.gelin.games.javatris.game;

import java.util.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;

/**
 *  Represents one record item, including user name, score etc.
 */
public class RecordItem implements Comparable {

    private static final SimpleDateFormat moDateFormat = new SimpleDateFormat("yyyy-MM-dd");

    /** Player name */
    private String msName;
    /**
     *  Returns name of record owner.
     *  @return player name
     */
    public String getName() {
        return msName;
    }
    /**
     *  Sets name of record owner.
     *  @param  asName  name of new user
     *  @throws NullPointerException    if parameter is null
     */
    public void setName(String asName) throws NullPointerException {
        if (asName == null) {
            throw new NullPointerException("asName can't be null");
        }
        msName = asName;
    }

    /** Player's score */
    private int miScore;
    /**
     *  Returns score of this record.
     *  @return score of this record item
     */
    public int getScore() {
        return miScore;
    }
    /**
     *  Sets score for this record item.
     *  @param  aiScore new score value
     */
    public void setScore(int aiScore) {
        miScore = aiScore;
    }

    /** Date of this record */
    private Date moDate;
    /**
     *  Returns date of this record.
     *  @return date of this record
     */
    public Date getDate() {
        return moDate;
    }
    /**
     *  Returns date of this record in string format "yyyy-MM-dd".
     *  @return date of this record
     */
    public String getDateAsString() {
        return moDateFormat.format(getDate());
    }
    /**
     *  Sets date of this record.
     *  @param  aoDate  date of this record
     *  @throws NullPointerException    if parameter is null
     */
    public void setDate(Date aoDate) throws NullPointerException {
        if (aoDate == null) {
            throw new NullPointerException("aoDate can't be null");
        }
        moDate = aoDate;
    }
    /**
     *  Sets date of this record as string in format "yyyy-MM-dd".
     *  @param  asDate  date of this record
     *  @throws NullPointerException    if parameter is null
     *  @throws ParseException      if parameter can't be parsed (misformatted)
     */
    public void setDateAsString(String asDate)
            throws NullPointerException, ParseException {
        if (asDate == null) {
            throw new NullPointerException("asDate can't be null");
        }
        setDate(moDateFormat.parse(asDate));
    }

    /**
     *  Constructor with name and score.
     *  @param  asName  name of the owner of new record
     *  @param  aiScore score of new record
     */
    public RecordItem(String asName, int aiScore) {
        setName(asName);
        setScore(aiScore);
        setDate(new Date());    //current date/time
    }

    /**
     *  Default constructor. Creates record of <i>Anonymous</i> with zero score and current date.
     */
    public RecordItem() {
        this("Anonymous", 0);
    }

    /**
     *  Compares this record with record specified by argument.
     *  @see    Comparable
     */
    public int compareTo(Object aoObj) {
        int result = 0;
        if (miScore < ((RecordItem)aoObj).miScore) {
            result = 1;
        } else if (miScore > ((RecordItem)aoObj).miScore) {
            result = -1;
        } else {
            result = msName.compareTo(((RecordItem)aoObj).msName);
            if (result == 0) {
                result = moDate.compareTo(((RecordItem)aoObj).moDate);
            }
        }
        return result;
    }

    /**
     *  Returns <code>true</code> if this item equals with <code>aoObj</code>
     */
    public boolean equals(Object aoObj) {
        return (aoObj instanceof RecordItem) &&
                (miScore == ((RecordItem)aoObj).miScore) &&
                (msName.equals(((RecordItem)aoObj).msName)) &&
                (moDate.equals(((RecordItem)aoObj).moDate));
    }

    /**
     *  Returns "canonical" string representation of this item. Used for checking of consistency
     *  of records list.
     *  @return concatenation of string representations of all fields
     */
    public String asString() {
        StringBuffer sBuffer = new StringBuffer();
        sBuffer.append(getName());
        sBuffer.append(getScore());
        sBuffer.append(moDateFormat.format(getDate()));
        return sBuffer.toString();
    }

    public String toString() {
        StringBuffer sBuffer = new StringBuffer();
        sBuffer.append("( ");
        sBuffer.append(getName());
        sBuffer.append(" : ");
        sBuffer.append(getScore());
        sBuffer.append(" : ");
        sBuffer.append(moDateFormat.format(getDate()));
        sBuffer.append(" )");
        return sBuffer.toString();
    }

}