/*
 *  JAJI - Just Another Javatris Implementation.
 *  Copyright (C) 2005  Denis Nelubin aka Gelin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  http://gelin.ru
 *  mailto:java@gelin.ru
 *
 *  2006-01-25  Gelin   Initially created
 */

package ru.gelin.games.javatris.game;

import ru.gelin.games.javatris.ui.ILabelModel;
import ru.gelin.games.javatris.ui.LabelEvent;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collections;
import javax.swing.SwingConstants;
import org.apache.commons.codec.digest.DigestUtils;

/**
 *  List of game records. Used as a part of game {@link Settings}.
 *  Provides way to validate records from illegal modifications (simple signature).
 *  Provides UI models to show best player.
 */
public class RecordsList {

    /** maximum number of saved record */
    public static final int SIZE = 10;

    /** secret for calculating signature */
    private static final String SECRET = "JAJIRecordsListSecret-FoolishThing";

    /** list of real records */
    private List moRecords = new ArrayList(SIZE);

    /** temporary list of records which may be put to the list */
    private List moToPut = new ArrayList();

    /** model of best score for UI */
    private ScoreModel moScoreModel = new ScoreModel();

    /** model of best player's name */
    private PlayerModel moPlayerModel = new PlayerModel();

    /**
     *  Default constructor.
     */
    public RecordsList() {
    }

    /**
     *  Returns real number of saved records.
     *  @return number of records
     */
    public int size() {
        return moRecords.size();
    }

    /**
     *  Returns record item with specified index.
     *  Record with zero index is best player's record.
     *  @param  aiIndex index between 0 and <code>size()</code>
     *  @return record
     *  @throws IndexOutOfBoundsException   if index is out of range
     */
    public RecordItem get(int aiIndex) {
        return (RecordItem)moRecords.get(aiIndex);
    }

    /**
     *  Returns iterator over records list.
     *  @return iterator
     */
    public Iterator iterator() {
        return Collections.unmodifiableList(moRecords).iterator();
    }

    /**
     *  Puts new record to list.
     *  Record is NOT put into the list immediately, one of the {@link #commit}
     *  methods must be called after some puts for real inserting.
     *  @param  aoItem  new record to be inserted to the list
     */
    public void put(RecordItem aoItem) {
        moToPut.add(aoItem);
    }

    /**
     *  Unconditionally inserts new records added by {@link #put} methods before
     *  to the list.
     *  Note that list is limited in size and sorted, if new record is worse than any
     *  already contained in list, nothing is inserted.
     *  @return <code>true</code> if list is changed, <code>false</code> if nothing is changed
     */
    public boolean commit() {
        boolean bResult = false;
        List oTempList = new ArrayList(moRecords);
        oTempList.addAll(moToPut);
        Collections.sort(oTempList);
        oTempList = oTempList.subList(0, SIZE);
        bResult = !(moRecords.containsAll(oTempList));
        if (bResult) {
            moRecords = oTempList;
            moToPut.clear();
            //fire IU model events
            moScoreModel.fireLabelEvent(new LabelEvent(moScoreModel, moScoreModel.getText()));
            moPlayerModel.fireLabelEvent(new LabelEvent(moPlayerModel, moPlayerModel.getText()));
        }
        return bResult;
    }

    /**
     *  Inserts new records added by {@link #put} methods before only if
     *  there signature is match with presented one.
     *  @param  asSign  signature of new records for checking
     *  @return <code>true</code> if signature is right and records list is updated,
     *          otherwise <code>false</code>
     */
    public boolean commit(String asSign) {
        boolean bResult = false;
        String oSign = getSign(moToPut);
        if (oSign.equals(asSign)) {
            bResult = commit();
        }
        return bResult;
    }

    /**
     *  Calculates signature of the list.
     *  @return string representation of signature
     */
    public String getSign() {
        return getSign(moRecords);
    }

    /**
     *  Calculates signature of the specified list.
     *  Used to calculate signature of real list or of temporary "to put" list.
     *  @param  aoList  list to calculate for
     *  @return signature
     */
    private String getSign(List aoList) {
        StringBuffer oBuf = new StringBuffer();
        oBuf.append(SECRET);
        Iterator i = aoList.iterator();
        while (i.hasNext()) {
            RecordItem oItem = (RecordItem)i.next();
            oBuf.append(oItem.asString());
        }
        return DigestUtils.md5Hex(oBuf.toString());  // MD5 digest from string using Jakarta Commons Codec
    }

    /**
     *  Checks if specified score is a new record, i.e. can be inserted into records list.
     *  @param  aiScore score of pretender to new record
     *  @return <code>true</code> if record is better than worse record in the list
     */
    public boolean isNewRecord(int aiScore) {
        int iWorse = get(size() - 1).getScore();
        return aiScore > iWorse;
    }

    /**
     *  Returns model of best player's score for UI.
     *  @return model for UI
     */
    public ILabelModel getBestScoreModel() {
        return moScoreModel;
    }

    /**
     *  Returns model of best player's name for UI.
     *  @return model for UI
     */
    public ILabelModel getBestPlayerModel() {
        return moPlayerModel;
    }

    /**
     *  Implementaion of top score model.
     */
    private class ScoreModel extends ALabelModel {
        public String getText() {
            if (moRecords.isEmpty()) {
                return "0";
            } else {
                return String.valueOf(((RecordItem)moRecords.get(0)).getScore());
            }
        }
        public int getHorizontalAlignment() {
            return SwingConstants.RIGHT;
        }
        //make method public - availiable to enclosing class
        public void fireLabelEvent(LabelEvent aoEvent) {
            super.fireLabelEvent(aoEvent);
        }
    }

    /**
     *  Implementaion of top player's name model.
     */
    private class PlayerModel extends ALabelModel {
        public String getText() {
            if (moRecords.isEmpty()) {
                return "Anonymous";
            } else {
                return ((RecordItem)moRecords.get(0)).getName();
            }
        }
        public int getHorizontalAlignment() {
            return SwingConstants.RIGHT;
        }
        //make method public - availiable to enclosing class
        public void fireLabelEvent(LabelEvent aoEvent) {
            super.fireLabelEvent(aoEvent);
        }
    }

}
