/*
 *  JAJI - Just Another Javatris Implementation.
 *  Copyright (C) 2005  Denis Nelubin aka Gelin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  http://gelin.ru
 *  mailto:java@gelin.ru
 *
 *  2006-01-10  Gelin   Initially created
 *  2006-09-11  Gelin   added mouse preferences
 */

package ru.gelin.games.javatris.game;

import java.awt.Color;
import java.awt.event.MouseEvent;

/**
 *  Represents game setting which can be changed in configuration dialogs and be saved
 *  to configuration file or cookie (for applet).
 */
public class Settings implements IGameLevels, IBlockColors {

    /** Level of complexity of new game */
    private int miNewLevel = LEVEL_ONE;
    /**
     *  Gets level of complexity of new game.
     *  @return level
     */
    public int getNewLevel() {
        return miNewLevel;
    }
    /**
     *  Sets level of complexity of new game.
     *  @param  aiNewLevel  level value
     */
    public void setNewLevel(int aiNewLevel) {
        if (aiNewLevel < MIN_LEVEL) {
            miNewLevel = MIN_LEVEL;
        } else if (aiNewLevel > MAX_LEVEL) {
            miNewLevel = MAX_LEVEL;
        } else {
            miNewLevel = aiNewLevel;
        }
    }

    /** Is it need to show next block ? */
    private boolean mbBlockPreview = true;
    /**
     *  Is it need to show next block ?
     *  @return <code>true</code> if next block should be shown
     */
    public boolean isBlockPreview() {
        return mbBlockPreview;
    }
    /**
     *  Sets block preview mode.
     *  @param  abBlockPreview  new flag value
     */
    public void setBlockPreview(boolean abBlockPreview) {
        mbBlockPreview = abBlockPreview;
    }

    /** Is default rotation direction clockwise ? */
    private boolean mbRotationClockwise = true;
    /**
     *  Is default block rotation direction clockwise ?
     *  @return <code>true</code> if rotation direction is clockwise, <code>false</code>
     *          if rotation direction is anticlockwise
     */
    public boolean isRotationClockwise() {
        return mbRotationClockwise;
    }
    /**
     *  Sets default block rotation direction.
     *  @param  abRotationClockwise <code>true</code> for clockwise rotation, <code>false</code>
     *                              for anticlockwise rotation
     */
    public void setRotationClockwise(boolean abRotationClockwise) {
        mbRotationClockwise = abRotationClockwise;
    }

    /** array of block and background colors (default values are defined) */
    private Color[] maColors = {
        Color.BLACK,        //BG_COLOR_!
        Color.DARK_GRAY,    //BG_COLOR_2
        Color.PINK,         //BLOCK_I
        Color.YELLOW,       //BLOCK_L
        Color.WHITE,        //BLOCK_IL
        Color.CYAN,         //BLOCK_T
        Color.BLUE,         //BLOCK_Z
        Color.GREEN,        //BLOCK_S
        Color.RED,          //BLOCK_SQ
    };
    /**
     *  Gets color by index.
     *  @param  aiColor color index between {@link IBlockColors#MIN_COLOR} and {@link IBlockColors#MAX_COLOR}
     *  @return color reference or <code>null</code> if index is out of bounds
     */
    public Color getColor(int aiColor) {
        Color oResult = null;
        if (aiColor >= MIN_COLOR && aiColor <= MAX_COLOR) {
            oResult = maColors[aiColor];
        }
        return oResult;
    }
    /**
     *  Sets color by index. Does nothing if color index is out of bounds.
     *  @param  aiColor color index between {@link IBlockColors#MIN_COLOR} and {@link IBlockColors#MAX_COLOR}
     *  @param  aoColor new color value
     */
    public void setColor(int aiColor, Color aoColor) {
        if (aiColor >= MIN_COLOR && aiColor <= MAX_COLOR) {
            maColors[aiColor] = aoColor;
        }
    }

    /** list of records */
    private RecordsList moRecords = new RecordsList();
    /* fill it with default values */
    {
        for (int i = 0; i < 10; i++) {
            moRecords.put(new RecordItem());
        }
        moRecords.commit();
    }
    /**
     *  Returns list of records.
     *  @return records list
     */
    public RecordsList getRecords() {
        return moRecords;
    }

    /** timer delays for each level */
    private int[] miDelays = { 0, 1000, 900, 800, 700, 600, 500, 400, 300, 200, 100 };
    /**
     *  Returns game timer delay for each game level.
     *  @param  miLevel game level
     *  @return timer delay in milliseconds
     */
    public int getTimerDelay(int miLevel) {
        return miDelays[miLevel];
    }

    /** drop timer delay. Drop must be fast. */
    private int miDropDelay = 0;
    /**
     *  Returns timer delay when dropping.
     *  @return timer delay in milliseconds
     */
    public int getDropDelay() {
        return miDropDelay;
    }

    /** scores for number of filled lines */
    private int[] miLineScores = { 0, 10, 100, 1000, 10000};
    /**
     *  Returns score for number of filled lines.
     *  @param  aiLines number of filled lines
     *  @return score increment
     */
    public int getScoreForLines(int aiLines) {
        return miLineScores[aiLines];
    }

    /** scores for new inserted block */
    private int miBlockScore = 5;
    /**
     *  Returns score for new inserting block
     *  @return scores for block
     */
    public int getScoreForBlock() {
        return miBlockScore;
    }

    /** scores for dropping block to one cell */
    private int miDropScore = 1;
    /**
     *  Returns score for dropping block to one cell
     *  @return score increment
     */
    public int getScoreForDrop() {
        return miDropScore;
    }

    /** number of lines which must be filled for go to next level */
    private int miLinesForLevel = 20;
    /**
     *  Returns number of lines which must be filled (and disappeared) to go to
     *  next game level.
     *  @return number of lines
     */
    public int getLinesForLevel() {
        return miLinesForLevel;
    }

    /** mouse enabled flag */
    private boolean mbMouseEnabled = true;
    /**
     *  Returns <code>true</code> if mouse should be used.
     */
    public boolean isMouseEnabled() {
        return mbMouseEnabled;
    }
    /**
     *  Sets value of mouse enabling flag.
     */
    public void setMouseEnabled(boolean abMouseEnabled) {
        mbMouseEnabled = abMouseEnabled;
    }

    /** mouse button which drops block */
    private int miDropMouseButton = MouseEvent.BUTTON1;
    /**
     *  Gets mouse button which drops block.
     */
    public int getDropMouseButton() {
        return miDropMouseButton;
    }
    /**
     *  Sets mouse button which drops block.
     */
    public void setDropMouseButton(int aiDropMouseButton) {
        miDropMouseButton = aiDropMouseButton;
    }

    /** mouse button which rotates block */
    private int miRotateMouseButton = MouseEvent.BUTTON3;
    /**
     *  Gets mouse button which rotates block.
     */
    public int getRotateMouseButton() {
        return miRotateMouseButton;
    }
    /**
     *  Sets mouse button which rotates block.
     */
    public void setRotateMouseButton(int aiRotateMouseButton) {
        miRotateMouseButton = aiRotateMouseButton;
    }

}