/*
 *  JAJI - Just Another Javatris Implementation.
 *  Copyright (C) 2005  Denis Nelubin aka Gelin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  http://gelin.ru
 *  mailto:java@gelin.ru
 *
 *  2005-10-06  Gelin   Initially created
 *  2006-06-11  Gelin   Added saving colors
 *  2006-09-16  Gelin   Added saving mouse control
 */

package ru.gelin.games.javatris.launch;

import ru.gelin.games.javatris.game.ISaver;
import ru.gelin.games.javatris.game.Settings;
import ru.gelin.games.javatris.game.RecordsList;
import ru.gelin.games.javatris.game.RecordItem;
import ru.gelin.games.javatris.game.IBlockColors;

import java.util.prefs.Preferences;
import java.util.prefs.BackingStoreException;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.text.ParseException;
import java.awt.Color;

/**
 *  Saves game state and setting to Java Preferences. Used for application.
 *  @see    java.util.prefs.Preferences
 */
public class PreferencesSaver implements ISaver {

    /** settings node */
    private static final String SETTINGS_NODE = "/ru/gelin/games/javatris/settings";

    /** new level key */
    private static final String NEW_LEVEL_KEY = "new_level";

    /** block preview key */
    private static final String BLOCK_PREVIEW_KEY = "block_preview";

    /** rotation clockwise key */
    private static final String ROTATION_CLOCKWISE_KEY = "rotation_clockwise";

    /** enable mouse key */
    private static final String MOUSE_ENABLED_KEY = "mouse_enabled";

    /** drop mouse button key */
    private static final String DROP_MOUSE_BUTTON_KEY = "drop_mouse_button";

    /** rotate mouse button key */
    private static final String ROTATE_MOUSE_BUTTON_KEY = "rotate_mouse_button";

    /** color key prefix */
    private static final String COLOR_KEY_PREFIX = "color_";

    /** records node */
    private static final String RECORDS_NODE = "/ru/gelin/games/javatris/records";

    /** record node prefix */
    private static final String RECORD_NODE_PREFIX = "record_";

    /** name key */
    private static final String NAME_KEY = "name";

    /** date key */
    private static final String DATE_KEY = "date";

    /** score key */
    private static final String SCORE_KEY = "score";

    /** signature key */
    private static final String SIGN_KEY = "sign";

    /** logger name */
    static public final String LOGGER_NAME = "ru.gelin.games.javatris.launch.PreferencesSaver";
    /** default log level */
    static public final Level DEF_LOG_LEVEL = Level.WARNING;
    /** logger instance */
    static private Logger moLog = Logger.getLogger(LOGGER_NAME);

    /**
     *  Saves game setting to User Preferences.
     */
    public void save(Settings aoSettings) {
        Preferences oPref = Preferences.userRoot().node(SETTINGS_NODE);
        oPref.putInt(NEW_LEVEL_KEY, aoSettings.getNewLevel());
        oPref.putBoolean(BLOCK_PREVIEW_KEY, aoSettings.isBlockPreview());
        oPref.putBoolean(ROTATION_CLOCKWISE_KEY, aoSettings.isRotationClockwise());
        oPref.putBoolean(MOUSE_ENABLED_KEY, aoSettings.isMouseEnabled());
        oPref.putInt(DROP_MOUSE_BUTTON_KEY, aoSettings.getDropMouseButton());
        oPref.putInt(ROTATE_MOUSE_BUTTON_KEY, aoSettings.getRotateMouseButton());
        for (int i = IBlockColors.MIN_COLOR; i <= IBlockColors.MAX_COLOR; i++) {
            oPref.putInt(COLOR_KEY_PREFIX + i, aoSettings.getColor(i).getRGB());
        }
    }

    /**
     *  Loads game settings from User Preferences.
     */
    public void load(Settings aoSettings) {
        Preferences oPref = Preferences.userRoot().node(SETTINGS_NODE);
        aoSettings.setNewLevel(oPref.getInt(NEW_LEVEL_KEY, aoSettings.getNewLevel()));
        aoSettings.setBlockPreview(oPref.getBoolean(BLOCK_PREVIEW_KEY, aoSettings.isBlockPreview()));
        aoSettings.setRotationClockwise(oPref.getBoolean(ROTATION_CLOCKWISE_KEY, aoSettings.isRotationClockwise()));
        aoSettings.setMouseEnabled(oPref.getBoolean(MOUSE_ENABLED_KEY, aoSettings.isMouseEnabled()));
        aoSettings.setDropMouseButton(oPref.getInt(DROP_MOUSE_BUTTON_KEY, aoSettings.getDropMouseButton()));
        aoSettings.setRotateMouseButton(oPref.getInt(ROTATE_MOUSE_BUTTON_KEY, aoSettings.getRotateMouseButton()));
        for (int i = IBlockColors.MIN_COLOR; i <= IBlockColors.MAX_COLOR; i++) {
            int iColor = oPref.getInt(COLOR_KEY_PREFIX + i, aoSettings.getColor(i).getRGB());
            if (iColor != aoSettings.getColor(i).getRGB()) {
                aoSettings.setColor(i, new Color(iColor));
            }
        }
    }

    /**
     *  Tries to save game records to System Preferences, if it's not possible,
     *  saves records to User Preferences.
     *  To access to System Preferences regular user must have write access to:
     *  <ul>
     *  <li><code>HKLM/Software/JavaSoft/Prefs</code> registry key in Windows</li>
     *  <li><code>$JRE_HOME/.systemPrefs</code> directory in Linux</li>
     *  <ul>
     */
    public void save(RecordsList aoRecordsList) {
        Preferences oPref = Preferences.systemRoot().node(RECORDS_NODE);
        boolean bExists = false;
        try {
            oPref.sync();   //!!! to detect system preference tree fail
            bExists = oPref.nodeExists("");
        } catch (BackingStoreException e) {
            moLog.logp(DEF_LOG_LEVEL, "PreferencesSaver", "save", "Can't create node '" + RECORDS_NODE + "' in system preference tree.", e);
        } finally {
            if (!bExists) {
                moLog.logp(DEF_LOG_LEVEL, "PreferencesSaver", "save", "Try to use user preference tree.");
                oPref = Preferences.userRoot().node(RECORDS_NODE);
            }
        }
        for (int i = 0; i < aoRecordsList.size(); i++) {
            RecordItem oRecord = aoRecordsList.get(i);
            Preferences oNode = oPref.node(RECORD_NODE_PREFIX + i);
            oNode.put(NAME_KEY, oRecord.getName());
            oNode.put(DATE_KEY, oRecord.getDateAsString());
            oNode.putInt(SCORE_KEY, oRecord.getScore());
        }
        oPref.put(SIGN_KEY, aoRecordsList.getSign());
    }

    /**
     *  Tries to load game records from System Preferences, if it's not possible,
     *  or records not exists, loads records from User Preferences.
     *  To access to System Preferences regular user must have read access to:
     *  <ul>
     *  <li><code>HKLM/Software/JavaSoft/Prefs</code> registry key in Windows</li>
     *  <li><code>$JRE_HOME/.systemPrefs</code> directory in Linux</li>
     *  <ul>
     */
    public void load(RecordsList aoRecordsList) {
        Preferences oPref = Preferences.systemRoot().node(RECORDS_NODE);
        boolean bExists = false;
        try {
            oPref.sync();   //!!! to detect system preference tree fail
            bExists = oPref.nodeExists("");
        } catch (BackingStoreException e) {
            moLog.logp(DEF_LOG_LEVEL, "PreferencesSaver", "load", "Not exists node '" + RECORDS_NODE + "' in system preference tree.", e);
        } finally {
            if (!bExists) {
                moLog.logp(DEF_LOG_LEVEL, "PreferencesSaver", "load", "Not exists node '" + RECORDS_NODE + "' in system preference tree. Try to use user preference tree.");
                oPref = Preferences.userRoot().node(RECORDS_NODE);
            }
        }
        int i = 0;
        try {
            for (i = 0; oPref.nodeExists(RECORD_NODE_PREFIX + i) && i < RecordsList.SIZE; i++) {
                String sDate = "";
                try {
                    RecordItem oRecord = new RecordItem();
                    Preferences oNode = oPref.node(RECORD_NODE_PREFIX + i);
                    oRecord.setName(oNode.get(NAME_KEY, oRecord.getName()));
                    sDate = oNode.get(DATE_KEY, oRecord.getDateAsString());
                    oRecord.setDateAsString(sDate);
                    oRecord.setScore(oNode.getInt(SCORE_KEY, oRecord.getScore()));
                    aoRecordsList.put(oRecord);
                } catch (ParseException e) {
                    moLog.logp(DEF_LOG_LEVEL, "PreferencesSaver", "load", "Invalid date format '" + sDate + "', record skipped.");
                }
            }
        } catch (BackingStoreException e) {
            moLog.logp(DEF_LOG_LEVEL, "PreferencesSaver", "load", "Not exists node '" + RECORD_NODE_PREFIX + i + "'.", e);
        }
        String sSign = oPref.get(SIGN_KEY, "empty records");
        if (!aoRecordsList.commit(sSign)) {
            moLog.logp(DEF_LOG_LEVEL, "PreferencesSaver", "load", "Records list with signature '" + sSign + "' is not loaded.");
        }
    }

}
