/*
 *  JAJI - Just Another Javatris Implementation.
 *  Copyright (C) 2005  Denis Nelubin aka Gelin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  http://gelin.ru
 *  mailto:java@gelin.ru
 *
 *  2006-06-11  Gelin   Initially created.
 */

package ru.gelin.games.javatris.ui;

import ru.gelin.games.javatris.game.IBlockColors;

import java.awt.Image;
import java.awt.Color;
import java.awt.Component;
import java.awt.image.ImageProducer;
import java.awt.image.MemoryImageSource;

/**
 *  Generates images showing blocks view in specified colors.
 *  Used to create attractive images on buttons which choose colors for blocks.
 */
public class BlockImageFactory implements IBlockColors {

    /** colors for draw images */
    Color[] moColors;

    /** width of glyph in cells */
    static final int GLYPH_WIDTH = 4;   //package access

    /** height of glyph in cells */
    static final int GLYPH_HEIGHT = 4;  //package access

    /** glyphs to produce images */
    static final int BLOCK_GLYPHS[][][] = { //package access
        //background 1
        {{ BG_COLOR_1, BG_COLOR_1, BG_COLOR_1, BG_COLOR_2 },
         { BG_COLOR_1, BG_COLOR_1, BG_COLOR_1, BG_COLOR_2 },
         { BG_COLOR_1, BG_COLOR_1, BG_COLOR_1, BG_COLOR_2 },
         { BG_COLOR_2, BG_COLOR_2, BG_COLOR_2, BG_COLOR_1 }},
        //background 2
        {{ BG_COLOR_2, BG_COLOR_2, BG_COLOR_2, BG_COLOR_1 },
         { BG_COLOR_2, BG_COLOR_2, BG_COLOR_2, BG_COLOR_1 },
         { BG_COLOR_2, BG_COLOR_2, BG_COLOR_2, BG_COLOR_1 },
         { BG_COLOR_1, BG_COLOR_1, BG_COLOR_1, BG_COLOR_2 }},
        //I-block
        {{ BG_COLOR_1, BLOCK_I,    BG_COLOR_1, BG_COLOR_2 },
         { BG_COLOR_2, BLOCK_I,    BG_COLOR_2, BG_COLOR_1 },
         { BG_COLOR_1, BLOCK_I,    BG_COLOR_1, BG_COLOR_2 },
         { BG_COLOR_2, BLOCK_I,    BG_COLOR_2, BG_COLOR_1 }},
        //L-block
        {{ BG_COLOR_1, BLOCK_L,    BG_COLOR_1, BG_COLOR_2 },
         { BG_COLOR_2, BLOCK_L,    BG_COLOR_2, BG_COLOR_1 },
         { BG_COLOR_1, BLOCK_L,    BLOCK_L,    BG_COLOR_2 },
         { BG_COLOR_2, BG_COLOR_1, BG_COLOR_2, BG_COLOR_1 }},
        //IL-block
        {{ BG_COLOR_1, BG_COLOR_2, BLOCK_IL,   BG_COLOR_2 },
         { BG_COLOR_2, BG_COLOR_1, BLOCK_IL,   BG_COLOR_1 },
         { BG_COLOR_1, BLOCK_IL,   BLOCK_IL,   BG_COLOR_2 },
         { BG_COLOR_2, BG_COLOR_1, BG_COLOR_2, BG_COLOR_1 }},
        //T-block
        {{ BG_COLOR_1, BG_COLOR_2, BG_COLOR_1, BG_COLOR_2 },
         { BLOCK_T,    BLOCK_T,    BLOCK_T,    BG_COLOR_1 },
         { BG_COLOR_1, BLOCK_T,    BG_COLOR_1, BG_COLOR_2 },
         { BG_COLOR_2, BG_COLOR_1, BG_COLOR_2, BG_COLOR_1 }},
        //Z-block
        {{ BG_COLOR_1, BG_COLOR_2, BG_COLOR_1, BG_COLOR_2 },
         { BLOCK_Z,    BLOCK_Z,    BG_COLOR_2, BG_COLOR_1 },
         { BG_COLOR_1, BLOCK_Z,    BLOCK_Z,    BG_COLOR_2 },
         { BG_COLOR_2, BG_COLOR_1, BG_COLOR_2, BG_COLOR_1 }},
        //S-block
        {{ BG_COLOR_1, BG_COLOR_2, BG_COLOR_1, BG_COLOR_2 },
         { BG_COLOR_2, BLOCK_S,    BLOCK_S,    BG_COLOR_1 },
         { BLOCK_S,    BLOCK_S,    BG_COLOR_1, BG_COLOR_2 },
         { BG_COLOR_2, BG_COLOR_1, BG_COLOR_2, BG_COLOR_1 }},
        //Square-block
        {{ BG_COLOR_1, BG_COLOR_2, BG_COLOR_1, BG_COLOR_2 },
         { BG_COLOR_2, BLOCK_SQ,   BLOCK_SQ,   BG_COLOR_1 },
         { BG_COLOR_1, BLOCK_SQ,   BLOCK_SQ,   BG_COLOR_2 },
         { BG_COLOR_2, BG_COLOR_1, BG_COLOR_2, BG_COLOR_1 }},
    };

    /** how many pixels need to use to draw one cell */
    static final int ZOOM = 6;  //package access

    /**
     *  Public constructor.
     *  @param  aoColors    array of colors in which images will be drawn,
     *                      nubmer of colors should be <code>IBlockColors.MAX_COLOR + 1</code>
     */
    public BlockImageFactory(Color[] aoColors) {
        moColors = aoColors;
    }

    /**
     *  Creates ImageProducer which displays image of block specified by index.
     *  @param  aiIndex index of block which image to render
     */
    public ImageProducer createImageProducer(int aiIndex) {
        int iWidth = GLYPH_WIDTH * ZOOM;
        int iHeight = GLYPH_HEIGHT * ZOOM;
        int[] iPixels = new int[iWidth * iHeight];
        for (int y = 0; y < iHeight; y++) {
            for (int x = 0; x < iWidth; x++) {
                int iColor = BLOCK_GLYPHS[aiIndex][y / ZOOM][x / ZOOM];
                iPixels[y * iWidth + x] = moColors[iColor].getRGB();
            }
        }
        return new MemoryImageSource(iWidth, iHeight, iPixels, 0, iWidth);
    }

    /**
     *  Creates image which displayes image of block, specified bu index.
     *  Image should be rendered on specified component.
     *  @param  aiIndex index of block which image to render
     *  @param  aoComponent component on which image to render
     */
    public Image createImage(int aiIndex, Component aoComponent) {
        return aoComponent.createImage(createImageProducer(aiIndex));
    }

}
