/*
 *  JAJI - Just Another Javatris Implementation.
 *  Copyright (C) 2005  Denis Nelubin aka Gelin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  http://gelin.ru
 *  mailto:java@gelin.ru
 *
 *  2006-06-11  Gelin   Initially created.
 */

package ru.gelin.games.javatris.ui;

import ru.gelin.games.javatris.game.IBlockColors;
import ru.gelin.games.javatris.game.AColorTableModel;
import ru.gelin.games.javatris.util.Messages;

import java.awt.Color;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JDialog;
import javax.swing.JColorChooser;
import javax.swing.JRootPane;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.BorderFactory;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

/**
 *  Dialog with JColorChooser and custom preview pane.
 */
public class ColorChooserDialog extends JDialog {

    /** color chooser */
    private JColorChooser moColorChooser;

    /** preview panel */
    private PreviewPanel moPreviewPanel;

    /** index of color to change */
    private int miColorIndex;

    /** array of current colors */
    private Color[] moColors = new Color[IBlockColors.MAX_COLOR + 1];

    /** OK button */
    private JButton moOkButton;

    /** OK button pressed flag */
    private boolean mbOkPressed = false;

    /** messages */
    private Messages moMsg = Messages.getInstance();

    /**
     *  Public constructor.
     */
    public ColorChooserDialog() {
        super((JFrame)null, true); //modal dialog without parent
        moColorChooser = new JColorChooser();
        moPreviewPanel = new PreviewPanel();
        moColorChooser.getSelectionModel().addChangeListener(moPreviewPanel);
        moColorChooser.setPreviewPanel(new JPanel());

        //bottom panel with buttons
        JPanel oBottomPanel = new JPanel();
        moOkButton = new JButton(moMsg.get("ok_button.text"));
        moOkButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    mbOkPressed = true;
                    hide();    //no dispose, dialog stays in memory for future showing
                }
            } );
        oBottomPanel.add(moOkButton);
        JButton oCancelButton = new JButton(moMsg.get("cancel_button.text"));
        oCancelButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    hide();    //no dispose, dialog stays in memory for future showing
                }
            } );
        oBottomPanel.add(oCancelButton);

        //add panels
        JPanel oTmpPanel = new JPanel(new BorderLayout());
        oTmpPanel.add(moColorChooser, BorderLayout.CENTER);
        oTmpPanel.add(moPreviewPanel, BorderLayout.SOUTH);
        getContentPane().add(oTmpPanel, BorderLayout.CENTER);
        getContentPane().add(oBottomPanel, BorderLayout.SOUTH);
        JRootPane oRoot = getRootPane();
        oRoot.setDefaultButton(moOkButton);
        oRoot.setWindowDecorationStyle(JRootPane.QUESTION_DIALOG);
        setDefaultCloseOperation(HIDE_ON_CLOSE);
        //pack();
    }

    /**
     *  Shows dialog for choosing color.
     *  @param  asTitle     title for dialog
     *  @param  aoColors    array with available colors
     *  @param  aiColorIndex    index of color which should be changed in array
     *  @return new Color if user selected colors and pressed OK, otherwise <code>null</code>
     */
    public Color chooseColor(String asTitle, Color[] aoColors, int aiColorIndex) {
        setTitle(asTitle);
        for (int i = IBlockColors.MIN_COLOR; i <= IBlockColors.MAX_COLOR; i++) {
            moColors[i] = aoColors[i];
        }
        miColorIndex = aiColorIndex;
        moColorChooser.setColor(moColors[miColorIndex]);
        pack();
        show(); //show dialog, wait for input
        if (mbOkPressed) {
            return moColorChooser.getColor();
        } else {
            return null;
        }
    }

    /**
     *  Color table panel with checking of color changing.
     */
    private class PreviewPanel extends ColorTable implements ChangeListener {

        public PreviewPanel() {
            super(new PreviewPanelModel());
            setBorder(BorderFactory.createLoweredBevelBorder());
        }

        /**
         *  When JColorChooser select new color this method is called.
         *  It updates local color and repaints PreviewPanel.
         */
        public void stateChanged(ChangeEvent e) {
            Color oNewColor = moColorChooser.getColor();
            moColors[miColorIndex] = oNewColor;
            repaint();
        }

    }

    /**
     *  Model for preview panel.
     *  Table of this model contains each glyphs of blocks from left to right
     */
    private class PreviewPanelModel extends AColorTableModel {

        /** number of glyphs (blocks) */
        private final int miNumGlyph = IBlockColors.MAX_BLOCK_TYPE - IBlockColors.MIN_BLOCK_TYPE + 1;

        public int getWidth() {
            return BlockImageFactory.GLYPH_WIDTH * miNumGlyph;
        }

        public int getHeight() {
            return BlockImageFactory.GLYPH_HEIGHT;
        }

        public int getMinimumCellSize() {
            return 3;
        }

        public int getPreferredCellSize() {
            return 15;
        }

        public int getMaximumCellSize() {
            return 100;
        }

        public Color getColor(int aiX, int aiY) {
            int iGlyph = aiX / BlockImageFactory.GLYPH_WIDTH + IBlockColors.MIN_BLOCK_TYPE;
            int iGlyphX = aiX % BlockImageFactory.GLYPH_WIDTH;
            int iColor = BlockImageFactory.BLOCK_GLYPHS[iGlyph][aiY][iGlyphX];
            return moColors[iColor];
        }

    }

}
