/*
 *  JAJI - Just Another Javatris Implementation.
 *  Copyright (C) 2005  Denis Nelubin aka Gelin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  http://gelin.ru
 *  mailto:java@gelin.ru
 *
 *  2005-10-24  Gelin   Initially created.
 *  2006-08-26  Gelin   Fixed bug #1
 */

package ru.gelin.games.javatris.ui;

import javax.swing.JComponent;
import javax.swing.border.Border;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

/**
 *  Shows colored table.
 *  @see    IColorTableModel
 */
public class ColorTable extends JComponent implements IColorTableListener {

    /** model */
    protected IColorTableModel moModel;

    /** current real cell size */
    protected int miCellSize;

    /** X shift used when not integer number of cells placed in table by width */
    protected int miXShift = 0;

    /** Y shift used when not integer number of cells placed in table by height */
    protected int miYShift = 0;

    /**
     *  Public constructor.
     *
     *  @param  aoModel model for color table
     */
    public ColorTable(IColorTableModel aoModel) {
        super();
        moModel = aoModel;
        moModel.addColorTableListener(this);
        updateSizes();
        //when size is changed, update real cell size
        addComponentListener(new ComponentAdapter() {
                    public void componentResized(ComponentEvent e) {
                        updateCellSize();
                    }
                });
    }

    /**
     *  Invokes when model changed.
     */
    public void colorTableChanged(ColorTableEvent aoEvent) {
        updateSizes();
        repaint(cellsToPixels(aoEvent.getRepaintRect()));
    }

    /**
     *  Paints component.
     */
    protected void paintComponent(Graphics g) {
        // find cells to repaint
        Rectangle oRect = pixelsToCells(g.getClipBounds());
        // draw not all cells
        // 2006-08-26 Gelin Bugfix #1 Why <= ??? But works!
        for (int x = oRect.x; x <= (oRect.x + oRect.width) && x < moModel.getWidth(); x++) {
            for (int y = oRect.y; y <= (oRect.y + oRect.height) && y < moModel.getHeight(); y++) {
                int iXCoord = miXShift + (x * miCellSize);
                int iYCoord = miYShift + (y * miCellSize);
                g.setColor(moModel.getColor(x, y));
                g.fillRect(iXCoord, iYCoord, miCellSize, miCellSize);
            }
        }
     }

    /**
     *  Paints components borders. Border of this component writes not
     *  on the edges of component, but on edges of drawn table.
     *  Code is modified from original source provided with JDK.
     */
    protected void paintBorder(Graphics g) {
        Border border = getBorder();
        if (border != null) {
            Insets oInsets = getInsets();
            border.paintBorder(this, g, miXShift - oInsets.left,
                    miYShift - oInsets.top,
                    miCellSize * moModel.getWidth() + oInsets.left + oInsets.right,
                    miCellSize * moModel.getHeight() + oInsets.top + oInsets.bottom);
        }
    }

    /**
     *  Recalculates components sizes when border is added.
     */
    public void setBorder(Border aoBorder) {
        super.setBorder(aoBorder);
        updateSizes();
    }

    /**
     *  Recalculates preferred, minimum and maximum component sizes.
     */
    protected void updateSizes() {
        Insets oInsets = getInsets();
        //preferred size
        int iWidth = moModel.getPreferredCellSize() * moModel.getWidth();
        int iHeight = moModel.getPreferredCellSize() * moModel.getHeight();
        //add border width
        iWidth += oInsets.left + oInsets.right;
        iHeight += oInsets.top + oInsets.bottom;
        setPreferredSize(new Dimension(iWidth, iHeight));
        //minimum size
        iWidth = moModel.getMinimumCellSize() * moModel.getWidth();
        iHeight = moModel.getMinimumCellSize() * moModel.getHeight();
        //add border width
        iWidth += oInsets.left + oInsets.right;
        iHeight += oInsets.top + oInsets.bottom;
        setMinimumSize(new Dimension(iWidth, iHeight));
        //maximum size
        iWidth = moModel.getMaximumCellSize() * moModel.getWidth();
        iHeight = moModel.getMaximumCellSize() * moModel.getHeight();
        //add border width
        iWidth += oInsets.left + oInsets.right;
        iHeight += oInsets.top + oInsets.bottom;
        setMaximumSize(new Dimension(iWidth, iHeight));
        //updateCellSize();
    }

    /**
     *  Recalculates current cell size
     */
    protected void updateCellSize() {
        //we have border
        Insets oInsets = getInsets();
        //determine current cell size
        int iCellWidth = (getWidth() - oInsets.left - oInsets.right) /
                 moModel.getWidth();
        int iCellHeight = (getHeight() - oInsets.top - oInsets.bottom) /
                 moModel.getHeight();
        miCellSize = (iCellWidth < iCellHeight) ? iCellWidth : iCellHeight;
        //calculate empty space
        miXShift = oInsets.left + (getWidth() - oInsets.left - oInsets.right -
                (miCellSize * moModel.getWidth())) / 2;
        miYShift = oInsets.top + (getHeight() - oInsets.top - oInsets.bottom -
                (miCellSize * moModel.getHeight())) / 2;
        //System.out.println("CellSize: " + miCellSize);
    }

    /**
     *  Converts rectangle in cell coordinates into rectangle in pixel coordinates.
     *  Resulting pixel rectangle fully includes cells from cell rectangle according
     *  current component state (size of cell, vertical and horizontal shift from beginning
     *  of coordinates).
     *  If parameter is <code>null</code>, method returns rectangle, which includes all
     *  component's pixels.
     *  @param  aoRect  rectangle in cells
     *  @return rectangle in pixels
     */
    protected Rectangle cellsToPixels(Rectangle aoRect) {
        Rectangle oResult = new Rectangle(0, 0, getWidth(), getHeight());   // all component rectangle
        if (aoRect != null) {
            oResult.x = miXShift + (aoRect.x * miCellSize);
            oResult.y = miYShift + (aoRect.y * miCellSize);
            oResult.width = aoRect.width * miCellSize;
            oResult.height = aoRect.height * miCellSize;
        }
        return oResult;
    }

    /**
     *  Converts rectangle in pixel coordinates into rectangle in cell coordinates.
     *  Resulting cell rectangle includes cells which may be only partly occupied by pixel rectangle.
     *  If parameter in <code>null</code>, method returns rectangle, which includes all model's cells.
     *  @param  aoRect  rectangle in pixels
     *  @return rectangle in cells
     */
    protected Rectangle pixelsToCells(Rectangle aoRect) {
        Rectangle oResult = new Rectangle(0, 0, moModel.getWidth(), moModel.getHeight());
        if (aoRect != null) {
            oResult.x = (aoRect.x - miXShift) / miCellSize;
            if (oResult.x < 0) {
                oResult.x = 0;
            }
            oResult.y = (aoRect.y - miYShift) / miCellSize;
            if (oResult.y < 0) {
                oResult.y = 0;
            }
            oResult.width = (aoRect.width + miCellSize - 1) / miCellSize;
            oResult.height = (aoRect.height + miCellSize - 1) / miCellSize;
        }
        return oResult;
    }

}
