/*
 *  JAJI - Just Another Javatris Implementation.
 *  Copyright (C) 2005  Denis Nelubin aka Gelin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  http://gelin.ru
 *  mailto:java@gelin.ru
 *
 *  2006-05-20  Gelin   Initially created
 *  2006-06-11  Gelin   Added choosing of colors, changed layout manager
 *  2006-09-11  Gelin   Added mouse properties
 */

package ru.gelin.games.javatris.ui;

import ru.gelin.games.javatris.game.Settings;
import ru.gelin.games.javatris.game.IGameLevels;
import ru.gelin.games.javatris.game.IBlockColors;
import ru.gelin.games.javatris.util.Messages;
import ru.gelin.games.javatris.util.StringIterator;

import java.util.Map;
import java.util.LinkedHashMap;
import java.util.Iterator;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.GridLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JRootPane;
import javax.swing.BorderFactory;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.Box;
import javax.swing.JComponent;
import javax.swing.ImageIcon;
import javax.swing.JColorChooser;
import javax.swing.JLabel;

/**
 *  Dialog with game settings.
 */
public class SettingsDialog extends JDialog {

    /** messages instance */
    private Messages moMsg = Messages.getInstance();

    /** game settings */
    private Settings moSettings;

    /** OK button */
    private JButton moOkButton;

    /** start level selection button group */
    private ButtonGroup moLevelsButtons;

    /** block preview checkbox */
    private JCheckBox moPreviewButton;

    /** rotation direction selection button group */
    private ButtonGroup moRotationButtons;

    /** max size for auto expanding panel */
    private int MAX_SIZE = Short.MAX_VALUE;

    /** array of colors, filled from settings */
    private Color[] moColors = new Color[IBlockColors.MAX_COLOR + 1];

    /** buttons which select color */
    private JButton[] moColorButtons = new JButton[IBlockColors.MAX_COLOR + 1];

    /** custom color chooser dialog */
    private ColorChooserDialog moColorChooser = null;

    /** mouse enable checkbox */
    private JCheckBox moMouseEnableCheckbox;

    /** drop mouse button group */
    private ButtonGroup moMouseDropButtons;

    /** rotate mouse button group */
    private ButtonGroup moMouseRotateButtons;

    /** map of mouse controls (for easier access) */
    private Map moMouseControls = new LinkedHashMap();

    /**
     *  Constructs dialog.
     *  @param  aoSettings  game settings
     */
    public SettingsDialog(Settings aoSettings) {
        super((JFrame)null, true); //modal dialog without parent
        setTitle(moMsg.get("setup_dialog.title"));
        moSettings = aoSettings;

        //content pane
        JPanel oPanel = new JPanel(new BorderLayout());

        //bottom panel with buttons
        JPanel oBottomPanel = new JPanel();
        moOkButton = new JButton(moMsg.get("ok_button.text"));
        moOkButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    saveSettings();
                    dispose();
                }
            } );
        oBottomPanel.add(moOkButton);
        JButton oCancelButton = new JButton(moMsg.get("cancel_button.text"));
        oCancelButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    dispose();
                }
            } );
        oBottomPanel.add(oCancelButton);

        //panel with new game levels
        JComponent oLevelsPanel = Box.createVerticalBox();
        oLevelsPanel.setBorder(BorderFactory.createTitledBorder(moMsg.get("setup_dialog.levels_border.title")));
        moLevelsButtons = new ButtonGroup();
        for (int i = IGameLevels.MIN_LEVEL; i <= IGameLevels.MAX_LEVEL; i++) {
            JRadioButton oButton = new JRadioButton(moMsg.get("setup_dialog.level_" + i + ".text"));
            oButton.setSelected(moSettings.getNewLevel() == i);
            oButton.setActionCommand(String.valueOf(i));    //for detecting selection after
            moLevelsButtons.add(oButton);
            oLevelsPanel.add(oButton);
        }

        //panel with options
        JComponent oOptionsPanel = Box.createVerticalBox();
        oOptionsPanel.setBorder(BorderFactory.createTitledBorder(moMsg.get("setup_dialog.options_border.title")));
        moPreviewButton = new JCheckBox(moMsg.get("setup_dialog.preview_checkbox.text"), moSettings.isBlockPreview());
        oOptionsPanel.add(moPreviewButton);

        //panel with rotation directions
        JComponent oRotationPanel = Box.createVerticalBox();
        oRotationPanel.setBorder(BorderFactory.createTitledBorder(moMsg.get("setup_dialog.rotation_border.title")));
        moRotationButtons = new ButtonGroup();
        for (int i = 0; i < 2; i++) {
            boolean b = (i == 0);
            JRadioButton oButton = new JRadioButton(moMsg.get("setup_dialog.clockwise_" + b + ".text"));
            oButton.setSelected(moSettings.isRotationClockwise() == b);
            oButton.setActionCommand(String.valueOf(b));    //for detecting selection after
            moRotationButtons.add(oButton);
            oRotationPanel.add(oButton);
        }

        //panel with mouse properties
        JComponent oMousePanel = Box.createVerticalBox();
        oMousePanel.setBorder(BorderFactory.createTitledBorder(moMsg.get("setup_dialog.mouse_border.title")));
        moMouseEnableCheckbox = new JCheckBox(moMsg.get("setup_dialog.enable_mouse.text"),
                !moSettings.isMouseEnabled());   //NOTE: We switch state later
        moMouseEnableCheckbox.addItemListener(new ItemListener() {    //listener disables mouse controls if mouse is disabled
                public void itemStateChanged(ItemEvent e) {
                    Iterator it = moMouseControls.keySet().iterator();
                    while (it.hasNext()) {
                        String sKey = (String) it.next();
                        JComponent oComponent = (JComponent) moMouseControls.get(sKey);
                        oComponent.setEnabled(e.getStateChange() == ItemEvent.SELECTED);
                    }
                }
            });
        oMousePanel.add(moMouseEnableCheckbox);
        JLabel oLabel = new JLabel(moMsg.get("setup_dialog.drop_mouse_button.label"));
        oMousePanel.add(oLabel);
        moMouseControls.put("drop_label", oLabel);
        moMouseDropButtons = new ButtonGroup();
        createMouseButtons(moMouseDropButtons, "drop_button_", moSettings.getDropMouseButton());
        ActionListener oListener = new MouseButtonActionListener("rotate_button_");
        Iterator it = StringIterator.startsWith(moMouseControls.keySet(), "drop_button_");
        while (it.hasNext()) {
            String sKey = (String) it.next();
            JRadioButton oButton = (JRadioButton) moMouseControls.get(sKey);
            oButton.addActionListener(oListener);   //for not use the same mouse button for drop and rotate
            oMousePanel.add(oButton);
        }
        oLabel = new JLabel(moMsg.get("setup_dialog.rotate_mouse_button.label"));
        oMousePanel.add(oLabel);
        moMouseControls.put("rotate_label", oLabel);
        moMouseRotateButtons = new ButtonGroup();
        createMouseButtons(moMouseRotateButtons, "rotate_button_", moSettings.getRotateMouseButton());
        oListener = new MouseButtonActionListener("drop_button_");
        it = StringIterator.startsWith(moMouseControls.keySet(), "rotate_button_");
        while (it.hasNext()) {
            String sKey = (String) it.next();
            JRadioButton oButton = (JRadioButton) moMouseControls.get(sKey);
            oButton.addActionListener(oListener);   //for not use the same mouse button for drop and rotate
            oMousePanel.add(oButton);
        }
        moMouseEnableCheckbox.doClick();    //change state and produce action event

        //panel with colors choosing buttons
        oListener = new ColorButtonActionListener();
        JComponent oColorsPanel = new JPanel();
        JComponent oTmpPanel = new JPanel(new GridLayout(5, 2, 5, 5));
        oColorsPanel.setBorder(BorderFactory.createTitledBorder(moMsg.get("setup_dialog.colors_border.title")));
        for (int i = IBlockColors.MIN_COLOR; i <= IBlockColors.MAX_COLOR; i++) {
            moColors[i] = moSettings.getColor(i);
            moColorButtons[i] = new JButton();
            moColorButtons[i].setToolTipText(moMsg.get("setup_dialog.color_" + i + ".tooltip"));
            moColorButtons[i].setActionCommand(String.valueOf(i));
            moColorButtons[i].addActionListener(oListener);
            oTmpPanel.add(moColorButtons[i]);
        }
        updateColorIcons();
        oColorsPanel.add(oTmpPanel);

        //add panels
        oPanel.add(oBottomPanel, BorderLayout.SOUTH);
        GridBagLayout oBag = new GridBagLayout();
        JPanel oMainPane = new JPanel(oBag);
        GridBagConstraints oConst = new GridBagConstraints();
        //panel with levels
        oConst.gridx = 0;
        oConst.gridy = 0;
        oConst.weightx = 0.5f;
        oConst.weighty = 0.0f;
        oConst.gridheight = 3;
        oConst.fill = GridBagConstraints.BOTH;
        oBag.setConstraints(oLevelsPanel, oConst);
        oMainPane.add(oLevelsPanel);
        //panel with options
        oConst.gridx = 1;
        oConst.gridy = 0;
        oConst.weightx = 0.5f;
        oConst.weighty = 0.0f;
        oConst.gridheight = 1;
        //oConst.anchor = GridBagConstraints.NORTH;
        oConst.fill = GridBagConstraints.HORIZONTAL;
        oBag.setConstraints(oOptionsPanel, oConst);
        oMainPane.add(oOptionsPanel);
        //panel with rotation
        oConst.gridx = 1;
        oConst.gridy = 1;
        oConst.fill = GridBagConstraints.HORIZONTAL;
        oBag.setConstraints(oRotationPanel, oConst);
        oMainPane.add(oRotationPanel);
        //panel with mouse properties
        oConst.gridx = 1;
        oConst.gridy = 2;
        oConst.weighty = 1.0f;
        oConst.fill = GridBagConstraints.BOTH;
        oBag.setConstraints(oMousePanel, oConst);
        oMainPane.add(oMousePanel);
        //panel with colors
        oConst.gridx = 2;
        oConst.gridy = 0;
        oConst.weighty = 1.0f;
        oConst.gridheight = 3;
        oConst.fill = GridBagConstraints.BOTH;
        oBag.setConstraints(oColorsPanel, oConst);
        oMainPane.add(oColorsPanel);

        oPanel.add(oMainPane, BorderLayout.CENTER);

        //set final parameters
        setContentPane(oPanel);
        JRootPane oRoot = getRootPane();
        oRoot.setDefaultButton(moOkButton);
        oRoot.setWindowDecorationStyle(JRootPane.QUESTION_DIALOG);
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
    }

    /**
     *  Shows dialog. This method do all nessesary work, so you do not need to call pack() method.
     *  Just create dialog and show() it.
     */
    public void show() {
        pack();
        moOkButton.requestFocusInWindow();  //this method is overidded for this call can be possible
        super.show();
    }

    /**
     *  Sets settings properties according current GUI elements position.
     */
    private void saveSettings() {
        //new game level
        moSettings.setNewLevel(Integer.parseInt(moLevelsButtons.getSelection().getActionCommand()));
        //block preview
        moSettings.setBlockPreview(moPreviewButton.isSelected());
        //rotation direction
        moSettings.setRotationClockwise(Boolean.valueOf(moRotationButtons.getSelection().getActionCommand()).booleanValue());
        //mouse control
        moSettings.setMouseEnabled(moMouseEnableCheckbox.isSelected());
        moSettings.setDropMouseButton(Integer.parseInt(moMouseDropButtons.getSelection().getActionCommand()));
        moSettings.setRotateMouseButton(Integer.parseInt(moMouseRotateButtons.getSelection().getActionCommand()));
        //colors
        for (int i = IBlockColors.MIN_COLOR; i <= IBlockColors.MAX_COLOR; i++) {
            moSettings.setColor(i, moColors[i]);
        }
    }

    /**
     *  Updates icons on color choosing buttons.
     */
    private void updateColorIcons() {
        BlockImageFactory oImages = new BlockImageFactory(moColors);
        for (int i = IBlockColors.MIN_COLOR; i <= IBlockColors.MAX_COLOR; i++) {
            moColorButtons[i].setIcon(new ImageIcon(oImages.createImage(i, moColorButtons[i])));
        }
    }

    /**
     *  Creates radio buttons with mouse button names.
     *  @param  aoButtonGroup   button group where to insert created radio buttons
     *  @param  asPrefix    prefix for map key where to store created radio buttons
     *  @param  aiSelected  integer value of selected mouse button in this group
     */
    private void createMouseButtons(ButtonGroup aoButtonGroup, String asPrefix, int aiSelected) {
        int[] aiButtons = { MouseEvent.BUTTON1, MouseEvent.BUTTON3 };
        for (int i = 0; i < aiButtons.length; i++) {
            int iButton = aiButtons[i];
            JRadioButton oButton = new JRadioButton(moMsg.get("setup_dialog.mouse_button_" + iButton + ".text"));
            oButton.setSelected(aiSelected == iButton);
            oButton.setActionCommand(String.valueOf(iButton));    //for detecting selection after
            aoButtonGroup.add(oButton);
            moMouseControls.put(asPrefix + String.valueOf(iButton), oButton);
        }
    }

    /**
     *  Frees resources of this dialog.
     */
    public void dispose() {
        if (moColorChooser != null) {
            moColorChooser.dispose();
        }
        super.dispose();
     }

    /**
     *  This class is used to react to pressing to color choosing buttons.
     */
    private class ColorButtonActionListener implements ActionListener {

        public void actionPerformed(ActionEvent oEvent) {
            int iColor = Integer.parseInt(oEvent.getActionCommand());
            String sTitle = moMsg.get("setup_dialog.color_dialog.title",
                    new Object[] { ((JButton)oEvent.getSource()).getToolTipText() });
            //Color oColor = JColorChooser.showDialog(SettingsDialog.this, sTitle, moColors[iColor]);
            if (moColorChooser == null) {
                moColorChooser = new ColorChooserDialog();
            }
            Color oColor = moColorChooser.chooseColor(sTitle, moColors, iColor);
            if (oColor != null) {
                moColors[iColor] = oColor;
                updateColorIcons();
            }
        }

    }

    /**
     *  This class reacts to choosing mouse buttons. It checks for condition that the same button
     *  can't be used for drop and rotate at the same time.
     */
    private class MouseButtonActionListener implements ActionListener {

        private String msPrefix;

        /**
         *  Constructor.
         *  @param  asPrefix    prefix for map keys of other button group
         */
        public MouseButtonActionListener(String asPrefix) {
            msPrefix = asPrefix;
        }

        public void actionPerformed(ActionEvent oEvent) {
            int iButton = Integer.parseInt(oEvent.getActionCommand());  //mouse button identifier
            JRadioButton oNextButton = null;    //next button will be selected
            boolean bMatch = false;  //no button matches
            Iterator it = StringIterator.startsWith(moMouseControls.keySet(), msPrefix);
            while (it.hasNext()) {
                String sKey = (String) it.next();
                JRadioButton oButton = (JRadioButton) moMouseControls.get(sKey);
                int iOtherButton = Integer.parseInt(oButton.getActionCommand());
                //System.out.println("Check for: " + sKey + " " + oButton);
                if (iButton == iOtherButton && oButton.isSelected()) {
                    bMatch = true;  //button match
                    if (it.hasNext()) { //not last button
                        oNextButton = (JRadioButton) moMouseControls.get((String) it.next());   //save next button
                        break;
                    }
                } else {
                    if (oNextButton == null) {  //if next button is not choosed
                        oNextButton = oButton;  //select first fitting
                    }
                }
            }
            if (bMatch && oNextButton != null) {
                oNextButton.setSelected(true);  //select next (other) button
            }
        }

    }

}
