/*
 *  JAJI - Just Another Javatris Implementation.
 *  Copyright (C) 2005  Denis Nelubin aka Gelin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  http://gelin.ru
 *  mailto:java@gelin.ru
 *
 *  2005-10-19  Gelin   Initially created
 */

package ru.gelin.games.javatris.util;

import java.util.Map;
import java.util.HashMap;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.net.URL;
import javax.swing.ImageIcon;

/**
 *  Produces all UI images.
 *  Images locations are taken from {@link ResourceBundle} with base in
 *  "ru.gelin.games.javatris.resources.images.IMG".
 */
public class Images {

    /** logger instance */
    static private Logger moLog = null;
    /** logger name */
    static private final String LOGGER_NAME = "ru.gelin.games.javatris.util.Images";
    /** default log level */
    static private final Level DEF_LOG_LEVEL = Level.WARNING;

    /** Produced instances map. Keys - locales, values - instances. */
    static private Map mmapInstances = new HashMap();

    /** Prefix for resources containing Images. */
    static private final String RES_BASE = "ru.gelin.games.javatris.resources.images.IMG";

    /** Default prefix for image location. */
    static private final String IMG_PATH = "/ru/gelin/games/javatris/resources/images/";

    /** Locale for this instance. */
    private Locale moLocale = null;

    /** ResourceBundle for this instance */
    private ResourceBundle moBundle = null;

    /**
     *  Default protected constructor, empty.
     *  @see    #getInstance
     */
    protected Images() {
        //we don't like default constructor
    }

    /**
     *  Protected constructor for internal use.
     *  @see    #getInstance
     */
    protected Images(Locale aoLocale) {
        moLog = Logger.getLogger(LOGGER_NAME);
        try {
            moBundle = ResourceBundle.getBundle(RES_BASE, aoLocale);
        } catch(MissingResourceException e) {
            moLog.logrb(DEF_LOG_LEVEL, "Images", "constructor", RES_BASE,
                    "Can't load " + RES_BASE + " properties", e);
            //moLog.logp(DEF_LOG_LEVEL, "Images", "constructor", "Can't load " + RES_BASE + " properties");
            //moLog.log(DEF_LOG_LEVEL, "", e);
        }
    }

    /**
     *  Creates new Images instance for default locale.
     *  @return Images object for default locale
     */
    public static Images getInstance() {
        return getInstance(Locale.getDefault());
    }

    /**
     *  Creates new Images instance for specified locale.
     *  @param  aoLocale    locale for creating Messages
     *  @return Images object for specified locale
     */
    public static Images getInstance(Locale aoLocale) {
        if (mmapInstances.get(aoLocale) != null) {
            return (Images)mmapInstances.get(aoLocale);
        } else {
            Images oImg = new Images(aoLocale);
            mmapInstances.put(aoLocale, oImg);
            return oImg;
        }
    }

    /**
     *  Returns locale for current instance.
     *  @return Locale, used by current instance of Messages
     */
    public Locale getLocale() {
        return moLocale;
    }

    /**
     *  Returns image by name. If image is not found, returnes <code>null</code>.
     *  @param  asImgName   name of image to be selected
     *  @return found image or null
     */
    public ImageIcon get(String asImgName) {
        ImageIcon oResult = null;
        if (moBundle == null) {
            moLog.logrb(DEF_LOG_LEVEL, "Messages", "get", RES_BASE,
                    "Resources " + RES_BASE + " are not loaded.");
            return oResult;
        } else {
            try {
                String sPath = moBundle.getString(asImgName);
                if (!sPath.startsWith("/")) {
                    sPath = IMG_PATH + sPath;
                }
                oResult = createImageIcon(sPath);
            } catch (MissingResourceException e) {
                moLog.logrb(DEF_LOG_LEVEL, "Images", "get", RES_BASE,
                        "Image '" + asImgName + "' not found.");
            }
            return oResult;
        }
    }

    /**
     *  Returns an ImageIcon, or null if the path was invalid.
     *  <br><em>Original code got from Java Swing tutorials</em>
     */
    protected ImageIcon createImageIcon(String path) {
        URL imgURL = Images.class.getResource(path);
        if (imgURL != null) {
            return new ImageIcon(imgURL);
        } else {
            moLog.logp(DEF_LOG_LEVEL, "Images", "createImageIcon", "Couldn't find file: " + path);
            return null;
        }
    }

}

