/*
 *  JAJI - Just Another Javatris Implementation.
 *  Copyright (C) 2005  Denis Nelubin aka Gelin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  http://gelin.ru
 *  mailto:java@gelin.ru
 *
 *  2005-10-10  Gelin   Initially created
 *  2006-04-09  Gelin   Added ability to format messages
 */

package ru.gelin.games.javatris.util;

import java.util.Map;
import java.util.HashMap;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.io.UnsupportedEncodingException;
import java.text.MessageFormat;

/**
 *  Produces all UI messages.
 *  Messages are taken from {@link ResourceBundle} with base in
 *  "ru.gelin.games.javatris.resources.messages.MSG".
 */
public class Messages {

    /** logger instance */
    static private Logger moLog = null;
    /** logger name */
    static private final String LOGGER_NAME = "ru.gelin.games.javatris.util.Messages";
    /** default log level */
    static private final Level DEF_LOG_LEVEL = Level.WARNING;

    /** Produced instances map. Keys - locales, values - instances. */
    static private Map mmapInstances = new HashMap();

    /** Prefix for resources containing messages. */
    static private final String RES_BASE = "ru.gelin.games.javatris.resources.messages.MSG";

    /** String returns then message is not found. */
    static private final String NOT_FOUND = "NOMESSAGE";

    /** Charset of MSG.properties files */
    static private final String PROP_CHARSET = "UTF-8";

    /** Locale for this instance. */
    private Locale moLocale = null;

    /** ResourceBundle for this instance */
    private ResourceBundle moBundle = null;

    /**
     *  Default protected constructor, empty.
     *
     *  @see    #getInstance
     */
    protected Messages() {
        //we don't like default constructor
    }

    /**
     *  Protected constructor for internal use.
     *
     *  @see    #getInstance
     */
    protected Messages(Locale aoLocale) {
        moLog = Logger.getLogger(LOGGER_NAME);
        try {
            moBundle = ResourceBundle.getBundle(RES_BASE, aoLocale);
        } catch (MissingResourceException e) {
            moLog.logrb(DEF_LOG_LEVEL, "Messages", "constructor", RES_BASE,
                    "Can't load " + RES_BASE + " properties", e);
        }
    }

    /**
     *  Creates new Messages instance for default locale.
     *  @return Messages object for default locale
     */
    public static Messages getInstance() {
        return getInstance(Locale.getDefault());
    }

    /**
     *  Creates new Messages instance for specified locale.
     *  @param  aoLocale    locale for creating Messages
     *  @return Messages object for specified locale
     */
    public static Messages getInstance(Locale aoLocale) {
        if (mmapInstances.get(aoLocale) != null) {
            return (Messages)mmapInstances.get(aoLocale);
        } else {
            Messages oMsg = new Messages(aoLocale);
            mmapInstances.put(aoLocale, oMsg);
            return oMsg;
        }
    }

    /**
     *  Returns locale for current instance.
     *  @return Locale, used by current instance of Messages
     */
    public Locale getLocale() {
        return moLocale;
    }

    /**
     *  Returns message by name. If message is not found, message name is
     *  returned.
     *  @param  asMsgName   name of message to be selected
     *  @return found message string
     */
    public String get(String asMsgName) {
        String sResult = asMsgName;
        if (moBundle == null) {
            moLog.logrb(DEF_LOG_LEVEL, "Messages", "get", RES_BASE,
                    "Resources " + RES_BASE + " are not loaded.");
            return sResult;
        } else {
            try {
                sResult = moBundle.getString(asMsgName);
                sResult = decodeResourceString(sResult, PROP_CHARSET);
            } catch (MissingResourceException e) {
                moLog.logrb(DEF_LOG_LEVEL, "Messages", "get", RES_BASE,
                        "Message '" + asMsgName + "' not found.");
            } catch (UnsupportedEncodingException e) {
                moLog.logp(DEF_LOG_LEVEL, "Messages", "get",
                        "Charset '" + PROP_CHARSET + "' is not supported.");
                throw new RuntimeException(e);  //this must never occured, so it's runtime error
            }
            return sResult;
        }
    }

    /**
     *  Returns formatted message by name. If message is not found, message name is
     *  returned. Message is formatted by values from second argument
     *  @param  asMsgName   name of message to be selected
     *  @param  aoArguments values which will be inserted to "{0}", "{1}", etc. patterns
     *  @return found and formatted message string
     *  @see MessageFormat
     */
    public String get(String asMsgName, Object[] aoArguments) {
        String sResult = asMsgName;
        if (moBundle == null) {
            moLog.logrb(DEF_LOG_LEVEL, "Messages", "get", RES_BASE,
                    "Resources " + RES_BASE + " are not loaded.");
        } else {
            try {
                sResult = moBundle.getString(asMsgName);
                sResult = decodeResourceString(sResult, PROP_CHARSET);
            } catch (MissingResourceException e) {
                moLog.logrb(DEF_LOG_LEVEL, "Messages", "get", RES_BASE,
                        "Message '" + asMsgName + "' not found.");
            } catch (UnsupportedEncodingException e) {
                moLog.logp(DEF_LOG_LEVEL, "Messages", "get",
                        "Charset '" + PROP_CHARSET + "' is not supported.");
                throw new RuntimeException(e);  //this must never occured, so it's runtime error
            }
        }
        sResult = MessageFormat.format(sResult, aoArguments);
        return sResult;
    }

    /**
     *  Encode string got from {@link ResourceBundle} according to specified
     *  charset.
     *  <br>
     *  Method supposes that string consists not of Unicode characters, but
     *  of bytes - ISO-8859-1 encoded characters. It gets this bytes and
     *  then encode it with specified charset.
     *  @param  asString    string to decode
     *  @param  asCharset   charset for decoding
     *  @return decoded string
     *  @throws UnsupportedEncodingException    if <code>asCharset</code> is not valid
     *  @see    java.nio.charset.Charset
     *  @see    String#getBytes(String)
     *  @see    String#String(byte[], String)
     */
    public static String decodeResourceString(String asString,
            String asCharset) throws UnsupportedEncodingException {
        byte[] baBuf = asString.getBytes("ISO-8859-1");
        return new String(baBuf, asCharset);
    }

}

