/*
 *  JAJI - Just Another Javatris Implementation.
 *  Copyright (C) 2005  Denis Nelubin aka Gelin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  http://gelin.ru
 *  mailto:java@gelin.ru
 *
 *  2004-04-15  Gelin   Initially created
 */

package ru.gelin.games.javatris.util;

import java.util.logging.Logger;
import java.util.logging.Level;
import java.net.URL;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.UnsupportedEncodingException;

/**
 *  Represents resource which can be found in classpath and extracted with different methods.
 */
public class Resource {

    /** path to resource */
    private String msPath;

    /** logger name */
    static private final String LOGGER_NAME = "ru.gelin.games.javatris.util.Resource";
    /** default log level */
    static private final Level DEF_LOG_LEVEL = Level.WARNING;
    /** logger instance */
    static private Logger moLog = Logger.getLogger(LOGGER_NAME);

    /** buffer size for string reading */
    static private final int BUF_SIZE = 1024;

    /**
     *  Creates resource.
     *  @param  asPath  absolute path to resource beginning from classpath
     */
    public Resource(String asPath) {
        msPath = asPath;
    }

    /**
     *  Gets resource's URL. You can use this URL directly or can use other
     *  convenience methods of this class.
     *  @return resource's URL or <code>null</code> if resource can't be found
     */
    public URL getURL() {
        URL oResult = null;
        oResult = Resource.class.getResource(msPath);
        if (oResult == null) {
            moLog.logp(DEF_LOG_LEVEL, "Resource", "getURL", "Couldn't find file: " + msPath);
        }
        return oResult;
    }

    /**
     *  Opens resource as input stream. You can use returned stream directly or can use
     *  other convenience methods of this class.
     *  Returned stream is not buffered, so it's better if you wrap it into BufferedInputStream.
     *  @return stream or <code>null</code> if resource can't be found
     *  @throws IOException if resource can't be open.
     *  @see    java.io.BufferedInputStream
     */
    public InputStream getStream() throws IOException {
        InputStream oResult = null;
        URL oUrl = getURL();
        if (oUrl != null) {
            oResult = oUrl.openStream();
        }
        return oResult;
    }

    /**
     *  Opens resource as Reader. You can use returned reader directly or can use
     *  other convenience methods of this class.
     *  Returned reader is not buffered, so it's betterif you wrap it into BufferedReader.
     *  @param  asCharSet   character set in which resource is encoded
     *  @return reader or <code>null</code> if resource can't be found
     *  @throws UnsupportedEncodingException    if specified charset is not supported
     *  @throws IOException if resource can't be open
     *  @see    java.nio.charset.Charset
     *  @see    java.io.BufferedReader
     */
    public Reader getReader(String asCharSet) throws UnsupportedEncodingException, IOException {
        Reader oResult = null;
        InputStream oStream = getStream();
        if (oStream != null) {
            oResult = new InputStreamReader(oStream, asCharSet);
        }
        return oResult;
    }

    /**
     *  Get resource content as string.
     *  @param  asCharSet   character set in which resource is encoded
     *  @return string with resource content or <code>null</code> if resource can't be found or read
     *  @throws UnsupportedEncodingException    if specified charset is not supported
     */
    public String getString(String asCharSet) throws UnsupportedEncodingException {
        String sResult = null;
        try {
            Reader oReader = getReader(asCharSet);
            if (oReader != null) {
                oReader = new BufferedReader(oReader);
                char[] cBuf = new char[BUF_SIZE];
                StringBuffer sBuf = new StringBuffer();
                int iRead = 0;  //number of read characters
                    while ((iRead = oReader.read(cBuf)) > 0) {
                        sBuf.append(cBuf, 0, iRead);
                    }
                sResult = sBuf.toString();
                oReader.close();
            }
        } catch (IOException e) {
            moLog.logp(DEF_LOG_LEVEL, "Resource", "getString", "IOExeption while reading " + msPath, e);
        }
        return sResult;
    }

}

