/*
 *  JAJI - Just Another Javatris Implementation.
 *  Copyright (C) 2005  Denis Nelubin aka Gelin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  http://gelin.ru
 *  mailto:java@gelin.ru
 *
 *  2006-09-16  Gelin   Initially created
 */

package ru.gelin.games.javatris.util;

import java.util.Iterator;
import java.util.Collection;
import java.util.NoSuchElementException;

/**
 *  Iterator which iterates through collection of <code>String</code>s
 *  using some match rules for skip some elements in collection.
 */
public class StringIterator implements Iterator {

    /**
     *  Creates <code>Iterator</code> which returns only elements which starts with specified string.
     *  @param  aoCollection    collection on which to iterate
     *  @param  asPrefix    prefix from which element should be started
     */
    public static Iterator startsWith(Collection aoCollection, String asPrefix) {
        return new StartsWithIterator(aoCollection.iterator(), asPrefix);
    }

    /** wrapped iterator */
    protected Iterator moIterator;

    /** next element */
    protected Object moNext;

    /**
     *  Protected constructor.
     *  @param  aoIterator  wrapped iterator
     */
    protected StringIterator(Iterator aoIterator) {
        moIterator = aoIterator;
    }

    public boolean hasNext() {
        if (moNext == null) {   //deffered initialization
            moNext = findNext();
        }
        return moNext != null;
    }

    public Object next() throws NoSuchElementException {
        if (moNext == null) {   //deffered initialization
            moNext = findNext();
        }
        Object oResult = moNext;
        if (moNext == null) {
            throw new NoSuchElementException();
        }
        moNext = findNext();
        return oResult;
    }

    /**
     *  Finds next matched element in wrapped iterator
     */
    private Object findNext() {
        Object oResult = null;
        while (!match(oResult) && moIterator.hasNext()) {   //finding
            oResult = moIterator.next();
        }
        if (!match(oResult)) {
            oResult = null;
        }
        return oResult;
    }

    public void remove() throws UnsupportedOperationException, IllegalStateException {
        throw new UnsupportedOperationException();  //remove is not implemented
    }

    public String toString() {
        return getClass().getName() + "[" + "iterator=" + moIterator +
                ",iterator.hasNext=" + moIterator.hasNext() +
                ",next=" + moNext +
                ",match(next)=" + match(moNext) + "]";
    }

    /**
     *  Checks next candidate element for some conditions. For <code>null</code> parameter MUST return <code>false</code>.
     *  This implementation checks for <code>null</code>.
     *  @param  aoObject    object which is checked
     */
    protected boolean match(Object aoObject) {
        return aoObject != null;
    }

    /**
     *  Iterator which returns only elements which starts with specified string.
     */
    protected static class StartsWithIterator extends StringIterator {

        protected String msPrefix;

        /**
         *  Constructor.
         *  @param  aoIterator  wrapped iterator
         *  @param  asPrefix    prefix from which element should be started
         */
        public StartsWithIterator(Iterator aoIterator, String asPrefix) {
            super(aoIterator);
            msPrefix = asPrefix;
        }

        /**
         *  Returns <code>true</code> if next element starts with predefined prefix.
         */
        protected boolean match(Object aoObject) {
            return (aoObject != null && aoObject.toString().startsWith(msPrefix));
        }

    }

}

